package uoc.tfg.gui.views.document;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;

import uoc.tfg.model.document.DocumentField;
import uoc.tfg.model.textanalysis.classification.Category;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

public class DocumentTableModel implements Serializable {
	
	private static final long serialVersionUID = -5867422892988446869L;
	private Document document;
	private StringProperty url = new SimpleStringProperty();
    private StringProperty title = new SimpleStringProperty();
    private StringProperty author = new SimpleStringProperty();
    private StringProperty date = new SimpleStringProperty();
    private StringProperty keyword = new SimpleStringProperty();
    private StringProperty content = new SimpleStringProperty();
    private StringProperty subject = new SimpleStringProperty();
    private StringProperty description = new SimpleStringProperty();
    private StringProperty format = new SimpleStringProperty();
    private StringProperty language = new SimpleStringProperty();
    private StringProperty location = new SimpleStringProperty();
    private StringProperty categoryName = new SimpleStringProperty();
    private Category category;
    private ObservableList<DocumentFieldTableModel> fieldModels = FXCollections.observableArrayList();
    
    
	public DocumentTableModel(Document document) {
		this.document = document;
		this.setUrl(document.get(DocumentField.URL.toString()));
		this.setTitle(document.get(DocumentField.TITLE.toString()));
		this.setAuthor(document.get(DocumentField.AUTHOR.toString()));
		this.setDate(document.get(DocumentField.DATE.toString()));
		this.setKeyword(document.get(DocumentField.KEYWORD.toString()));
		this.setContent(document.get(DocumentField.CONTENT.toString()));
		this.setSubject(document.get(DocumentField.SUBJECT.toString()));
		this.setDescription(document.get(DocumentField.DESCRIPTION.toString()));
		this.setFormat(document.get(DocumentField.FORMAT.toString()));
		this.setLanguage(document.get(DocumentField.LANGUAGE.toString()));
		this.setLocation(document.get(DocumentField.LOCATION.toString()));
		for(DocumentField df: DocumentField.values()) {
			this.fieldModels.add(
					new DocumentFieldTableModel(df.toString(), document.get(df.toString())));
		}
	}

    
	public final Document getDocument() {
		return document;
	}

	public final void setDocument(Document document) {
		this.document = document;
	}


	public final String getUrl() {
		return url.get();
	}


	public final void setUrl(String value) {
		this.url.set(value);
	}

	public StringProperty urlProperty() {
    	return url;
    }

	public final String getTitle() {
		return title.get();
	}


	public final void setTitle(String value) {
		this.title.set(value);
	}

	public StringProperty titleProperty() {
    	return title;
    }

	public final String getAuthor() {
		return author.get();
	}


	public final void setAuthor(String value) {
		this.author.set(value);
	}

	public StringProperty authorProperty() {
    	return author;
    }

	public final String getDate() {
		return date.get();
	}


	public final void setDate(String value) {
		this.date.set(value);
	}

	public StringProperty dateProperty() {
    	return date;
    }

	public final String getKeyword() {
		return keyword.get();
	}


	public final void setKeyword(String value) {
		this.keyword.set(value);
	}

	public StringProperty keywordProperty() {
    	return keyword;
    }

	public final String getContent() {
		return content.get();
	}


	public final void setContent(String value) {
		this.content.set(value);
	}

	public StringProperty contentProperty() {
    	return content;
    }

	public final String getSubject() {
		return subject.get();
	}


	public final void setSubject(String value) {
		this.subject.set(value);
	}

	public StringProperty subjectProperty() {
    	return subject;
    }

	public final String getDescription() {
		return description.get();
	}


	public final void setDescription(String value) {
		this.description.set(value);
	}

	public StringProperty descriptionProperty() {
    	return description;
    }

	public final String getFormat() {
		return format.get();
	}


	public final void setFormat(String value) {
		this.format.set(value);
	}

	public StringProperty formatProperty() {
    	return format;
    }

	public final String getLanguage() {
		return language.get();
	}


	public final void setLanguage(String value) {
		this.language.set(value);
	}

	public StringProperty languageProperty() {
    	return language;
    }

	public final String getLocation() {
		return location.get();
	}


	public final void setLocation(String value) {
		this.location.set(value);
	}
    
	public StringProperty locationProperty() {
    	return location;
    }
	
	public final String getCategoryName() {
		return categoryName.get();
	}

	public void setCategoryName(String value) {
		this.categoryName.set(value);
	}
	
	public StringProperty categoryNameProperty() {
		return categoryName;
	}
	
	public ObservableList<DocumentFieldTableModel> getFieldModels() {
		return fieldModels;
	}


	public void setFieldModels(ObservableList<DocumentFieldTableModel> fieldModels) {
		this.fieldModels = fieldModels;
	}


	public Category getCategory() {
		return category;
	}


	public void setCategory(Category category) {
		this.category = category;
	}

}
