package uoc.tfg.gui.views.marketdata;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.marketdata.MarketdataLSInvoke;
import uoc.tfg.jpa.marketdata.LightstreamerJPA;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.paint.Color;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.util.Callback;

public class StreamingAdapterViewController implements Initializable {

	@FXML
	private ResourceBundle resources;

	@FXML
	private URL location;
	
	@FXML
    private Button dataProviderSelectionButton;

    @FXML
    private TextField dataProviderTextField;

    @FXML
    private TableColumn<MarketDataTableModel, String> maxColumn;

    @FXML
    private TableColumn<MarketDataTableModel, String> minColumn;

    @FXML
    private Button pauseButton;

    @FXML
    private TableColumn<MarketDataTableModel, String> priceColumn;

    @FXML
    private Button resumeButton;

    @FXML
    private Button startButton;

    @FXML
    private Button stopButton;

    @FXML
    private TableColumn<MarketDataTableModel, String> symbolColumn;

    @FXML
    private TableColumn<MarketDataTableModel, String> timeColumn;

    @FXML
    private TableColumn<MarketDataTableModel, String> varpColumn;

    @FXML
    private TableColumn<MarketDataTableModel, String> varvColumn;

    @FXML
    private TableColumn<MarketDataTableModel, String> volColumn;
    
    @FXML
    private TableView<MarketDataTableModel> tableView;

    private MarketdataLSInvoke dataInvoke = new MarketdataLSInvoke();
    
    private LightstreamerJPA server = new LightstreamerJPA();
	
	
	@Override
    public void initialize(URL url, ResourceBundle rb) {
		
		this.symbolColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().symbolProperty();
    		}
    	});
		
		this.priceColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().priceProperty();
    		}
    	});
		
		this.varpColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().varPProperty();
    		}
    	});
		
		this.varvColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().varVProperty();
    		}
    	});
		
		this.maxColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().maxProperty();
    		}
    	});
		
		this.minColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().minProperty();
    		}
    	});
		
		this.volColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().volumenProperty();
    		}
    	});
		
		this.timeColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<MarketDataTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<MarketDataTableModel, String> p) {
    			return p.getValue().dateProperty();
    		}
    	});
		
		this.tableView.setItems(this.dataInvoke.getEntries());
    }
    
	
	@FXML
	void actionDataProviderSelectionButton(ActionEvent event) {
		
		Stage stage = new Stage();
    	stage.setTitle("Select streaming server");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/marketdata/SelectStreamingConnectionView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectStreamingConnectionViewController controller = 
				fxmlLoader.<SelectStreamingConnectionViewController>getController();
    	controller.initialize(this.server, this.dataProviderTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
	}
	
	@FXML
	void actionStartButton(ActionEvent event) {
		try {
			this.dataInvoke.startMarketDataStreaming();
		} catch (Exception e) {
			new MessageDialog(e);
		}
	}
	
	@FXML
	void actionPauseButton(ActionEvent event) {
	}
	
	@FXML
	void actionResumeButton(ActionEvent event) {
	}
	
	@FXML
	void actionStopButton(ActionEvent event) {
		try {
			this.dataInvoke.stopMarketDataStreaming();
		} catch (Exception e) {
			new MessageDialog(e);
		}
	}
	
}	
	