/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.tfg.gui.views.webcrawling;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import uoc.tfg.exception.GuiException;
import uoc.tfg.exception.WebCrawlingException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.gui.views.document.SelectIndexViewController;
import uoc.tfg.invoke.webcrawling.WebCrawlerInvoke;
import uoc.tfg.invoke.webcrawling.WebCrawlingStrategyInvoke;
import uoc.tfg.model.document.Index;
import uoc.tfg.model.news.WebContentSimpleParser;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;
import uoc.tfg.model.news.webcrawler.WebCrawlingSeed;
import uoc.tfg.model.news.webcrawler.WebCrawlingSimpleStrategy;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.util.Callback;

/**
 * FXML Controller class
 *
 * @author jsanzol
 */
public class WebCrawlerViewController implements Initializable {

	@FXML
    private ResourceBundle resources;

    @FXML
    private URL location;
    
    @FXML
    private Button acceptButton;

    @FXML
    private ComboBox<?> algorithmComboBox;

    @FXML
    private Button cancelButton;

    @FXML
    private TextField delayTextField;

    @FXML
    private TextField depthTextField;

    @FXML
    private TextArea descriptionStrategyTextArea;

    @FXML
    private TableColumn<WebCrawlingEntryTableModel, String> entryDescriptionColumn;

    @FXML
    private TableColumn<WebCrawlingEntryTableModel, String> entryTitleColumn;

    @FXML
    private TableView<WebCrawlingEntryTableModel> entryTableView;

    @FXML
    private TableColumn<WebCrawlingEntryTableModel, String> entryUpdateColumn;

    @FXML
    private TextField criteria1TextField;

    @FXML
    private TextField criteria2TextField;

    @FXML
    private TextField criteria3TextField;

    @FXML
    private ComboBox<?> field1ComboBox;

    @FXML
    private ComboBox<?> field2ComboBox;

    @FXML
    private ComboBox<?> field3ComboBox;

    @FXML
    private ComboBox<?> logical1ComboBox;

    @FXML
    private ComboBox<?> logical2ComboBox;

    @FXML
    private ComboBox<?> logical3ComboBox;

    @FXML
    private TextField maxURLsTextField;

    @FXML
    private TextField nameStrategyTextField;

    @FXML
    private Button pauseButton;

    @FXML
    private Button resumeButton;

    @FXML
    private TextField seedTextField;

    @FXML
    private Button selectSeedButton;

    @FXML
    private Button selectStrategyButton;

    @FXML
    private Button startButton;

    @FXML
    private Text statusText;

    @FXML
    private Button stopButton;

    @FXML
    private TextField strategyTextField;
    
    @FXML
    private TextField indexTextField;

    @FXML
    private Text visitedUrlsText;

    @FXML
    private WebView webView;
      
    
    private WebCrawlerInvoke webCrawlerInvoke;
    
    private IWebCrawlingStrategy webCrawlingStrategy = new WebCrawlingSimpleStrategy();
    
    private Index index = new Index();
    
    private WebCrawlingSeed webCrawlingSeed;    

    
    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
    
    	
    	this.entryTitleColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebCrawlingEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebCrawlingEntryTableModel, String> p) {
    			return p.getValue().titleProperty();
    		}
    	});
    	
    	this.entryUpdateColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebCrawlingEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebCrawlingEntryTableModel, String> p) {
    			return p.getValue().dateProperty();
    		}
    	});
    	
    	this.entryDescriptionColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebCrawlingEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebCrawlingEntryTableModel, String> p) {
    			return p.getValue().descriptionProperty();
    		}
    	});
    }    

    @FXML
    private void actionOnSelectEntry(MouseEvent event) {
    	
    	WebEngine webEngine = this.webView.getEngine();
    	webEngine.load(
    			this.entryTableView.getSelectionModel().getSelectedItem().getEntry().getUrl().toString());
    }
    
    
    @FXML
    void actionSelectStrategyButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select startegy");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/SelectWebCrawlingStrategyView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebCrawlingStrategyViewController controller = 
				fxmlLoader.<SelectWebCrawlingStrategyViewController>getController();
    	controller.initialize(this, this.webCrawlingStrategy, this.strategyTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
    @FXML
    void actionSelectIndexButton(ActionEvent event) {
    	
    	Stage stage = new Stage();
    	stage.setTitle("Select index");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/document/SelectIndexView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectIndexViewController controller = 
				fxmlLoader.<SelectIndexViewController>getController();
    	controller.initialize(this.index, this.indexTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
    @FXML
    void actionStartButton(ActionEvent event) {
    	try {
			this.webCrawlerInvoke = new WebCrawlerInvoke(this);
		} catch (IOException e1) {
			new MessageDialog(e1);
		}
    	this.entryTableView.setItems(this.webCrawlerInvoke.getEntries());
    	try {
			this.webCrawlerInvoke.startWebCrawler();
		} catch (WebCrawlingException e2) {
			new MessageDialog(e2);
		}
    }

    @FXML
    void actionStopButton(ActionEvent event) {
    	this.webCrawlerInvoke.stopWebCrawler();
    }

    @FXML
    void actionPauseButton(ActionEvent event) {
    	this.webCrawlerInvoke.pausetWebCrawler();
    }

    @FXML
    void actionResumeButton(ActionEvent event) {
    	this.webCrawlerInvoke.resumeWebCrawler();
    }
        
    
    @FXML
    private void actionSelectSeedButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select seed");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/SelectWebCrawlingSeedView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebCrawlingSeedViewController controller = 
				fxmlLoader.<SelectWebCrawlingSeedViewController>getController();
    	controller.initialize(this.webCrawlingSeed, this.seedTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }

    @FXML
    private void accionAcceptButton(ActionEvent event) {
    	String name = this.nameStrategyTextField.getText();
    	String description = this.descriptionStrategyTextArea.getText();
    	Integer maxUrls = Integer.parseInt(this.maxURLsTextField.getText());
    	Integer depth = Integer.parseInt(this.depthTextField.getText());
    	Long delay = Long.parseLong(this.delayTextField.getText());
    	String algo = (String) (this.algorithmComboBox.getValue());
    	
    	if(name.isEmpty() || description.isEmpty()) {
    		new MessageDialog(new GuiException(GuiException.FORM_NOT_COMPLETE));
    	} else {
    		
    		IWebCrawlingStrategy wcs = new WebCrawlingSimpleStrategy(
    				name, 
    				description, 
    				maxUrls.intValue(), 
    				depth.intValue(), 
    				algo, 
    				delay.longValue(), 
    				this.webCrawlingSeed, 
    				new WebContentSimpleParser());
    		
    		WebCrawlingStrategyInvoke.newWebCrawlingURL(wcs);
    	}
    	    	
    }

    @FXML
    private void actionCancelButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

	/**
	 * 
	 * @return WebCrawlingSeed
	 */
	public WebCrawlingSeed getWebCrawlingSeed() {
		return webCrawlingSeed;
	}

	/**
	 * @param WebCrawlingStrategyViewController
	 */
	public void setWebCrawlingSeed(WebCrawlingSeed webCrawlingSeed) {
		this.webCrawlingSeed = webCrawlingSeed;
		this.seedTextField.setText(this.webCrawlingSeed.getName());
	}
	
	private void setInterestCriteria() {
		
		String[] logicals = new String[3];
    	String[] fields = new String[3];
    	String[] queries = new String[3];
    	
    	if(this.logical1ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[0] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.logical2ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[1] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.logical3ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[2] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	
    	if(this.field1ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[0] = this.field1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.field2ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[1] = this.field2ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.field3ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[2] = this.field3ComboBox.getSelectionModel().getSelectedItem().toString();
    	
    	if(!this.criteria1TextField.getText().isEmpty())
    		queries[0] = this.criteria1TextField.getText();
    	if(!this.criteria2TextField.getText().isEmpty())
    		queries[1] = this.criteria2TextField.getText();
    	if(!this.criteria3TextField.getText().isEmpty())
    		queries[2] = this.criteria3TextField.getText();
    	
    	int i = 0;
    	for(i=0;i<3;i++) {
    		if(queries[i] == null) break;
    	}
    	
    	String[] logicalsCopy = new String[i];
    	String[] fieldsCopy = new String[i];
    	String[] queriesCopy = new String[i];
    	
    	System.arraycopy(logicals, 0, logicalsCopy, 0, i);
    	System.arraycopy(fields, 0, fieldsCopy, 0, i);
    	System.arraycopy(queries, 0, queriesCopy, 0, i);
	}

	public ComboBox<?> getAlgorithmComboBox() {
		return algorithmComboBox;
	}

	public void setAlgorithmComboBox(ComboBox<?> algorithmComboBox) {
		this.algorithmComboBox = algorithmComboBox;
	}

	public TextField getDelayTextField() {
		return delayTextField;
	}

	public void setDelayTextField(TextField delayTextField) {
		this.delayTextField = delayTextField;
	}

	public TextField getDepthTextField() {
		return depthTextField;
	}

	public void setDepthTextField(TextField depthTextField) {
		this.depthTextField = depthTextField;
	}

	public TextArea getDescriptionStrategyTextArea() {
		return descriptionStrategyTextArea;
	}

	public void setDescriptionStrategyTextArea(TextArea descriptionStrategyTextArea) {
		this.descriptionStrategyTextArea = descriptionStrategyTextArea;
	}

	public TextField getCriteria1TextField() {
		return criteria1TextField;
	}

	public void setCriteria1TextField(TextField criteria1TextField) {
		this.criteria1TextField = criteria1TextField;
	}

	public TextField getCriteria2TextField() {
		return criteria2TextField;
	}

	public void setCriteria2TextField(TextField criteria2TextField) {
		this.criteria2TextField = criteria2TextField;
	}

	public TextField getCriteria3TextField() {
		return criteria3TextField;
	}

	public void setCriteria3TextField(TextField criteria3TextField) {
		this.criteria3TextField = criteria3TextField;
	}

	public ComboBox<?> getField1ComboBox() {
		return field1ComboBox;
	}

	public void setField1ComboBox(ComboBox<?> field1ComboBox) {
		this.field1ComboBox = field1ComboBox;
	}

	public ComboBox<?> getField2ComboBox() {
		return field2ComboBox;
	}

	public void setField2ComboBox(ComboBox<?> field2ComboBox) {
		this.field2ComboBox = field2ComboBox;
	}

	public ComboBox<?> getField3ComboBox() {
		return field3ComboBox;
	}

	public void setField3ComboBox(ComboBox<?> field3ComboBox) {
		this.field3ComboBox = field3ComboBox;
	}

	public ComboBox<?> getLogical1ComboBox() {
		return logical1ComboBox;
	}

	public void setLogical1ComboBox(ComboBox<?> logical1ComboBox) {
		this.logical1ComboBox = logical1ComboBox;
	}

	public ComboBox<?> getLogical2ComboBox() {
		return logical2ComboBox;
	}

	public void setLogical2ComboBox(ComboBox<?> logical2ComboBox) {
		this.logical2ComboBox = logical2ComboBox;
	}

	public ComboBox<?> getLogical3ComboBox() {
		return logical3ComboBox;
	}

	public void setLogical3ComboBox(ComboBox<?> logical3ComboBox) {
		this.logical3ComboBox = logical3ComboBox;
	}

	public TextField getMaxURLsTextField() {
		return maxURLsTextField;
	}

	public void setMaxURLsTextField(TextField maxURLsTextField) {
		this.maxURLsTextField = maxURLsTextField;
	}

	public TextField getNameStrategyTextField() {
		return nameStrategyTextField;
	}

	public void setNameStrategyTextField(TextField nameStrategyTextField) {
		this.nameStrategyTextField = nameStrategyTextField;
	}

	public TextField getSeedTextField() {
		return seedTextField;
	}

	public void setSeedTextField(TextField seedTextField) {
		this.seedTextField = seedTextField;
	}

	public TextField getStrategyTextField() {
		return strategyTextField;
	}

	public void setStrategyTextField(TextField strategyTextField) {
		this.strategyTextField = strategyTextField;
	}

	public Text getVisitedUrlsText() {
		return visitedUrlsText;
	}

	public void setVisitedUrlsText(Text visitedUrlsText) {
		this.visitedUrlsText = visitedUrlsText;
	}

	public IWebCrawlingStrategy getWebCrawlingStrategy() {
		return webCrawlingStrategy;
	}

	public void setWebCrawlingStrategy(IWebCrawlingStrategy webCrawlingStrategy) {
		this.webCrawlingStrategy = webCrawlingStrategy;
	}

	public Index getIndex() {
		return index;
	}

	public void setIndex(Index index) {
		this.index = index;
	}

	public Text getStatusText() {
		return statusText;
	}

	public void setStatusText(Text statusText) {
		this.statusText = statusText;
	}
	
	
        
}
