/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.tfg.gui.views.webcrawling;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.naming.NamingException;

import uoc.tfg.exception.GuiException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.webcrawling.WebCrawlingSeedInvoke;
import uoc.tfg.model.news.webcrawler.WebCrawlingSeed;
import uoc.tfg.model.news.webcrawler.WebCrawlingURL;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.paint.Color;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 * FXML Controller class
 *
 * @author jsanzol
 */
public class WebCrawlingSeedViewController implements Initializable {

	@FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Button acceptButton;

    @FXML
    private Button addURLButton;

    @FXML
    private Button cancelButton;

    @FXML
    private TextArea descriptionSeedTextArea;

    @FXML
    private Button editURLButton;

    @FXML
    private TextField nameSeedTextField;

    @FXML
    private Button newURLButton;

    @FXML
    private Button removeURLButton;

    @FXML
    private TableView<WebCrawlingURL> seedUrlsTableView;

    @FXML
    private TableColumn<WebCrawlingURL, String> urlColumn;
    
    @FXML
    private TableColumn<WebCrawlingURL, String> nameColumn;

    @FXML
    private TableColumn<WebCrawlingURL, String> descriptionColumn;
    
    private boolean isNewMode = true;
    
    private WebCrawlingSeed webCrawlingSeed;
        
    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
    	nameColumn.setCellValueFactory(
    			new PropertyValueFactory<WebCrawlingURL, String>("name"));
    	urlColumn.setCellValueFactory(
    			new PropertyValueFactory<WebCrawlingURL, String>("urlText"));
		descriptionColumn.setCellValueFactory(
				new PropertyValueFactory<WebCrawlingURL, String>("description"));
    }    
    
    public void initialize(WebCrawlingSeed wcs, 
			TableView<WebCrawlingSeed> table, boolean newMode) {
    	nameColumn.setCellValueFactory(
				new PropertyValueFactory<WebCrawlingURL, String>("name"));
    	urlColumn.setCellValueFactory(
				new PropertyValueFactory<WebCrawlingURL, String>("urlText"));
    	descriptionColumn.setCellValueFactory(
				new PropertyValueFactory<WebCrawlingURL, String>("description"));
    	this.isNewMode = newMode;
		if(this.isNewMode) {
    		this.webCrawlingSeed = wcs;
    	} else {
    		this.webCrawlingSeed = wcs;
    		this.nameSeedTextField.setText(wcs.getName());
    		this.nameSeedTextField.setEditable(false);;
    		this.descriptionSeedTextArea.setText(wcs.getDescription());
        	this.seedUrlsTableView.getItems().addAll(wcs.getUrls());
    	}			
	}

    @FXML
    private void accionAcceptButton(ActionEvent event) {
    	String name = this.nameSeedTextField.getText();
    	String description = this.descriptionSeedTextArea.getText();
    	List<WebCrawlingURL> wcus = new ArrayList<WebCrawlingURL>();
    	for(WebCrawlingURL wcu: this.seedUrlsTableView.getItems()) {
    		wcus.add(wcu);
    	}    	 
    	
    	if(name.isEmpty() || description.isEmpty() || wcus.isEmpty()) {
    		new MessageDialog(new GuiException(GuiException.FORM_NOT_COMPLETE));
    	} else {
    		WebCrawlingSeed wcs = null;
			
    		if(this.isNewMode) {
    			try {
    				wcs = new WebCrawlingSeed( 
    						name, 
    						description,
    						wcus);
        			WebCrawlingSeedInvoke.newWebCrawlingURL(wcs);;
        		} catch (NamingException e) {
        			new MessageDialog(e);
        		}
    		} else {
    			try {
    				this.webCrawlingSeed.setName(name);
    				this.webCrawlingSeed.setDescription(description);
    				this.webCrawlingSeed.setURLs(wcus);;
    				WebCrawlingSeedInvoke.editWebCrawlingURL(wcs);
        		} catch (NamingException e) {
        			new MessageDialog(e);
        		}
    		} 		
    		
    		((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    	}
    }

    @FXML
    private void actionCancelButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

    @FXML
    private void actionAddURLButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select seed URL");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/SelectWebCrawlingURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebCrawlingURLViewController controller = 
				fxmlLoader.<SelectWebCrawlingURLViewController>getController();
    	controller.initialize(this.seedUrlsTableView);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }

    @FXML
    private void actionRemoveURLButton(ActionEvent event) {
    	this.seedUrlsTableView.getItems().remove(
    			this.seedUrlsTableView.getSelectionModel().getSelectedItem());
    }

    @FXML
    private void actionNewURLButton(ActionEvent event) {
    	WebCrawlingURL wcu = null;
		try {
			wcu = new WebCrawlingURL();
		} catch (MalformedURLException e) {
			new MessageDialog(e);
		}
    	Stage stage = new Stage();
    	stage.setTitle("New seed URL");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/WebCrawlingURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
    	WebCrawlingURLViewController controller = 
    			fxmlLoader.<WebCrawlingURLViewController>getController();
    	if(wcu != null) {
    		controller.initialize(wcu, this.seedUrlsTableView, true);
    	}
    	Scene scene = new Scene(root, 400, 300, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }

    @FXML
    private void actionEditURLButton(ActionEvent event) {
    	WebCrawlingURL wcu = this.seedUrlsTableView.getSelectionModel().getSelectedItem();
    	Stage stage = new Stage();
    	stage.setTitle("Edit seed URL");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/WebCrawlingURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
    	WebCrawlingURLViewController controller = 
    			fxmlLoader.<WebCrawlingURLViewController>getController();
    	controller.initialize(wcu, this.seedUrlsTableView, false);
    	Scene scene = new Scene(root, 400, 300, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
}
