/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.tfg.gui.views.webcrawling;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import uoc.tfg.exception.GuiException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.webcrawling.WebCrawlingStrategyInvoke;
import uoc.tfg.model.news.WebContentSimpleParser;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;
import uoc.tfg.model.news.webcrawler.WebCrawlingSeed;
import uoc.tfg.model.news.webcrawler.WebCrawlingSimpleStrategy;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.paint.Color;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 * FXML Controller class
 *
 * @author jsanzol
 */
public class WebCrawlingStrategyViewController implements Initializable {

	@FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Button acceptButton;

    @FXML
    private ComboBox<?> algorithmComboBox;

    @FXML
    private Button cancelButton;

    @FXML
    private TextField delayTextField;

    @FXML
    private TextField depthTextField;

    @FXML
    private TextArea descriptionStrategyTextArea;

    @FXML
    private TextField criteria1TextField;

    @FXML
    private TextField criteria2TextField;

    @FXML
    private TextField criteria3TextField;

    @FXML
    private ComboBox<?> field1ComboBox;

    @FXML
    private ComboBox<?> field2ComboBox;

    @FXML
    private ComboBox<?> field3ComboBox;

    @FXML
    private ComboBox<?> logical1ComboBox;

    @FXML
    private ComboBox<?> logical2ComboBox;

    @FXML
    private ComboBox<?> logical3ComboBox;

    @FXML
    private TextField maxURLsTextField;

    @FXML
    private TextField nameStrategyTextField;

    @FXML
    private TextField seedTextField;

    @FXML
    private Button selectSeedButton;
    
    private WebCrawlingSeed webCrawlingSeed = new WebCrawlingSeed();
    
    
    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        // TODO
    }    

    @FXML
    private void actionSelectSeedButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select seed");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webcrawling/SelectWebCrawlingSeedView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebCrawlingSeedViewController controller = 
				fxmlLoader.<SelectWebCrawlingSeedViewController>getController();
    	controller.initialize(this.webCrawlingSeed, this.seedTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }

    @FXML
    private void accionAcceptButton(ActionEvent event) {
    	String name = this.nameStrategyTextField.getText();
    	String description = this.descriptionStrategyTextArea.getText();
    	Integer maxUrls = Integer.parseInt(this.maxURLsTextField.getText());
    	Integer depth = Integer.parseInt(this.depthTextField.getText());
    	Long delay = Long.parseLong(this.delayTextField.getText());
    	String algo = (String) (this.algorithmComboBox.getValue());
    	
    	if(name.isEmpty() || description.isEmpty()) {
    		new MessageDialog(new GuiException(GuiException.FORM_NOT_COMPLETE));
    	} else {
    		
    		IWebCrawlingStrategy wcs = new WebCrawlingSimpleStrategy(
    				name, 
    				description, 
    				maxUrls.intValue(), 
    				depth.intValue(), 
    				algo, 
    				delay.longValue(), 
    				this.webCrawlingSeed, 
    				new WebContentSimpleParser());
    		
    		WebCrawlingStrategyInvoke.newWebCrawlingURL(wcs);
    	}
    	    	
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

    @FXML
    private void actionCancelButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

	/**
	 * 
	 * @return WebCrawlingSeed
	 */
	public WebCrawlingSeed getWebCrawlingSeed() {
		return webCrawlingSeed;
	}

	/**
	 * @param WebCrawlingStrategyViewController
	 */
	public void setWebCrawlingSeed(WebCrawlingSeed webCrawlingSeed) {
		this.webCrawlingSeed = webCrawlingSeed;
		this.seedTextField.setText(this.webCrawlingSeed.getName());
	}
	
	private void setInterestCriteria() {
		
		String[] logicals = new String[3];
    	String[] fields = new String[3];
    	String[] queries = new String[3];
    	
    	if(this.logical1ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[0] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.logical2ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[1] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.logical3ComboBox.getSelectionModel().getSelectedItem() != null)
    		logicals[2] = this.logical1ComboBox.getSelectionModel().getSelectedItem().toString();
    	
    	if(this.field1ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[0] = this.field1ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.field2ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[1] = this.field2ComboBox.getSelectionModel().getSelectedItem().toString();
    	if(this.field3ComboBox.getSelectionModel().getSelectedItem() != null)
    		fields[2] = this.field3ComboBox.getSelectionModel().getSelectedItem().toString();
    	
    	if(!this.criteria1TextField.getText().isEmpty())
    		queries[0] = this.criteria1TextField.getText();
    	if(!this.criteria2TextField.getText().isEmpty())
    		queries[1] = this.criteria2TextField.getText();
    	if(!this.criteria3TextField.getText().isEmpty())
    		queries[2] = this.criteria3TextField.getText();
    	
    	int i = 0;
    	for(i=0;i<3;i++) {
    		if(queries[i] == null) break;
    	}
    	
    	String[] logicalsCopy = new String[i];
    	String[] fieldsCopy = new String[i];
    	String[] queriesCopy = new String[i];
    	
    	System.arraycopy(logicals, 0, logicalsCopy, 0, i);
    	System.arraycopy(fields, 0, fieldsCopy, 0, i);
    	System.arraycopy(queries, 0, queriesCopy, 0, i);
	}
	
}
