/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */

package uoc.tfg.gui.views.webfeed;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.naming.NamingException;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.webfeed.WebFeedSubscriptionInvoke;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.paint.Color;
import javafx.stage.Modality;
import javafx.stage.Stage;

public class SearchWebFeedSubscriptionViewController implements Initializable {
	
	@FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Button createWebFeedReaderButton;

    @FXML
    private Button deleteButton;

    @FXML
    private Button editButton;

    @FXML
    private Button exitSearchWebFeedButton;

    @FXML
    private Button listAllWebFeedSubscriptionButton;

    @FXML
    private Button resetWebFeedResetButton;

    @FXML
    private Button searchWebFeedSubscriptionButton;

    @FXML
    private TextField searchWebFeedSubscriptionName;

    @FXML
    private TextField searchWebFeedSubscriptionURL;

    @FXML
    private TableColumn<WebFeedSubscription, String> subscriptionDescription;

    @FXML
    private TableColumn<WebFeedSubscription, String> subscriptionName;
    
    @FXML
    private TableView<WebFeedSubscription> table;

    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
    	subscriptionName.setCellValueFactory(
    			new PropertyValueFactory<WebFeedSubscription, String>("name"));
    	subscriptionDescription.setCellValueFactory(
    			new PropertyValueFactory<WebFeedSubscription, String>("description"));
    }   

    /**
     * 
     * @param event
     */
    @FXML
    void accionListAllWebFeedSubscriptionButton(ActionEvent event) {
    	try {
    		this.table.getItems().addAll(
    				WebFeedSubscriptionInvoke.getAllWebFeedSubscriptions());
		} catch (MalformedURLException | NamingException | DatabaseException e) {
			new MessageDialog(e);
		}
    	
    }

    /**
     * 
     * @param event
     */
    @FXML
    void accionSearchWebFeedSubscriptionButton(ActionEvent event) {
    	List<WebFeedSubscription> wfss = new ArrayList<WebFeedSubscription>();
    	try {
    		wfss.add(WebFeedSubscriptionInvoke.getWebFeedSubscription(
					this.searchWebFeedSubscriptionName.getText()));
		} catch (MalformedURLException | NamingException | DatabaseException e) {
			new MessageDialog(e);
		}
    	
    	this.table.getItems().clear();
    	this.table.getItems().addAll(wfss);
    }

    /**
     * 
     * @param event
     */
    @FXML
    void actionCreateWebFeedReaderButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Web feed reader");
    	Parent root = null;
		try {
			root = FXMLLoader.load(getClass().getResource(
					"/uoc/tfg/gui/views/webfeed/WebFeedReaderView.fxml"));
		} catch (IOException e) {
			new MessageDialog(e);
		}
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);    
    	stage.setScene(scene);
    	stage.show();
    	this.table.getSelectionModel().getSelectedItem();
    }

    /**
     * 
     * @param event
     */
    @FXML
    void actionDeleteButton(ActionEvent event) {
    	WebFeedSubscription wfs = this.table.getSelectionModel().getSelectedItem();
    	try {
			WebFeedSubscriptionInvoke.deleteWebFeedSubscription(wfs.getName());
		} catch (NamingException | DatabaseException e) {
			new MessageDialog(e);
		}
    	this.table.getItems().remove(wfs);
    }

    /**
     * 
     * @param event
     */
    @FXML
    void actionEditButton(ActionEvent event) {
    	WebFeedSubscription wfs = this.table.getSelectionModel().getSelectedItem();
    	Stage stage = new Stage();
    	stage.setTitle("Edit Web feed subscription");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webfeed/WebFeedSubscriptionView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
    	WebFeedSubscriptionViewController controller = 
    			fxmlLoader.<WebFeedSubscriptionViewController>getController();
    	controller.initialize(wfs, this.table, false);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }

    /**
     * 
     * @param event
     */
    @FXML
    void actionExitSearchWebFeedButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

    /**
     * 
     * @param event
     */
    @FXML
    void actionResetWebFeedResetButton(ActionEvent event) {
    	this.searchWebFeedSubscriptionName.clear();
    	this.searchWebFeedSubscriptionURL.clear();
    }

    
}
