/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.gui.views.webfeed;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import uoc.tfg.exception.GuiException;
import uoc.tfg.exception.WebFeedException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.gui.views.document.SelectIndexViewController;
import uoc.tfg.invoke.webfeed.WebFeedReaderInvoke;
import uoc.tfg.model.document.Index;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.util.Callback;


public class WebFeedReaderViewController implements Initializable {
	
	@FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private WebView contentPanel;

    @FXML
    private TextField webFeedSubscriptionTextField;
    
    @FXML
    private TextField indexTextField;
    
    @FXML
    private Button findWebFeedSubscriptionButton;

    @FXML
    private Button pauseWebFeedReaderButton;

    @FXML
    private Button resumeWebFeedReaderButton;
    
    @FXML
    private Button stopWebFeedReaderButton;

    @FXML
    private Button startWebFeedReaderButton;

    @FXML
    private TableColumn<WebFeedReaderInvoke, String> webFeedSubscriptionColumn;

    @FXML
    private TableColumn<WebFeedReaderInvoke, String> statusWebFeedColumn;

    @FXML
    private TableView<WebFeedReaderInvoke> webFeedSubscriptionsTable;

    @FXML
    private TableColumn<WebFeedEntryTableModel, String> webContentTitleColumn;

    @FXML
    private TableColumn<WebFeedEntryTableModel, String> webContentDescriptionColumn;

    @FXML
    private TableColumn<WebFeedEntryTableModel, String> webContentDateColumn;

    @FXML
    private TableView<WebFeedEntryTableModel> webContentTable;
    
    private Index index = new Index();
    

    @Override
    public void initialize(URL url, ResourceBundle rb) {
    	
    	this.webFeedSubscriptionColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebFeedReaderInvoke, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebFeedReaderInvoke, String> p) {
    			return p.getValue().getNameProperty();
    		}
    	});
    	
    	this.statusWebFeedColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebFeedReaderInvoke, String>, ObservableValue<String>>() {
    	     public ObservableValue<String> call(CellDataFeatures<WebFeedReaderInvoke, String> p) {
    	         return p.getValue().getStatusProperty();
    	     }
    	});
    	
    	this.webContentTitleColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebFeedEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebFeedEntryTableModel, String> p) {
    			return p.getValue().titleProperty();
    		}
    	});
    	
    	this.webContentDescriptionColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebFeedEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebFeedEntryTableModel, String> p) {
    			return p.getValue().descriptionProperty();
    		}
    	});
    	
    	this.webContentDateColumn.setCellValueFactory(
    			new Callback<CellDataFeatures<WebFeedEntryTableModel, String>, ObservableValue<String>>() {
    		public ObservableValue<String> call(CellDataFeatures<WebFeedEntryTableModel, String> p) {
    			return p.getValue().dateProperty();
    		}
    	});
    		
    }
    
    /**
     * 
     * @param event
     */
    @FXML
    private void actionOnSelectEntry(MouseEvent event) {
    	
    	WebEngine webEngine = this.contentPanel.getEngine();
    	webEngine.load(
    			this.webContentTable.getSelectionModel().getSelectedItem().getEntry().getLink());
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionOnSelectSubscription(MouseEvent event) {
    	this.webContentTable.setItems(
    			this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem().getEntries());
    	this.webFeedSubscriptionTextField.setText(
    			this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem().getNameProperty().get());
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionStartWebFeedReaderButton(ActionEvent event) {
    	WebFeedReaderInvoke fr = this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem();
    	if(fr != null) {
    		try {
    			fr.setIndex(this.index);
    			fr.startWebFeedReader();
    		} catch (WebFeedException | IOException e) {
    			new MessageDialog(e);
    		}   
    	} else {
    		new MessageDialog(new GuiException(GuiException.NOT_SELECTION));
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionPauseWebFeedReaderButton(ActionEvent event) {
    	WebFeedReaderInvoke fr = this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem();
    	if(fr != null) {
    		fr.pausetWebFeedReader();   
    	} else {
    		new MessageDialog(new GuiException(GuiException.NOT_SELECTION));
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionResumeWebFeedReaderButton(ActionEvent event) {
    	WebFeedReaderInvoke fr = this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem();
    	if(fr != null) {
    		fr.resumeWebFeedReader();;   
    	} else {
    		new MessageDialog(new GuiException(GuiException.NOT_SELECTION));
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionStopWebFeedReaderButton(ActionEvent event) {
    	WebFeedReaderInvoke fr = this.webFeedSubscriptionsTable.getSelectionModel().getSelectedItem();
    	if(fr != null) {
    		fr.stopWebFeedReader();;   
    	} else {
    		new MessageDialog(new GuiException(GuiException.NOT_SELECTION));
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionFindWebFeedSubscriptionButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select Web feed subscription");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webfeed/SelectWebFeedSubscriptionView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebFeedSubscriptionViewController controller = 
				fxmlLoader.<SelectWebFeedSubscriptionViewController>getController();
    	controller.initialize(this);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
    @FXML
    void actionSelectIndexButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select index");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/document/SelectIndexView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectIndexViewController controller = 
				fxmlLoader.<SelectIndexViewController>getController();
    	controller.initialize(this.index, this.indexTextField);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);   
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }


    /**
     * 
     * @return
     */
	public TableView<WebFeedReaderInvoke> getWebFeedSubscriptionsTable() {
		return webFeedSubscriptionsTable;
	}

	/**
	 * 
	 * @param webFeedSubscriptionsTable
	 */
	public void setWebFeedSubscriptionsTable(
			TableView<WebFeedReaderInvoke> webFeedSubscriptionsTable) {
		this.webFeedSubscriptionsTable = webFeedSubscriptionsTable;
	}
	
	public Index getIndex() {
		return index;
	}

	public void setIndex(Index index) {
		this.index = index;
	}
    
}
