/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.gui.views.webfeed;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.naming.NamingException;

import uoc.tfg.exception.GuiException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.webfeed.WebFeedSubscriptionInvoke;
import uoc.tfg.model.Interval;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;
import uoc.tfg.model.news.webfeed.WebFeedURL;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.paint.Color;
import javafx.stage.Modality;
import javafx.stage.Stage;


public class WebFeedSubscriptionViewController implements Initializable {

    @FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Button acceptButton;

    @FXML
    private Button addURLButton;
    
    @FXML
    private Button newURLButton;
    
    @FXML
    private Button removeURLButton;
    
    @FXML
    private Button editURLButton;

    @FXML
    private Button cancelButton;

    @FXML
    private TextArea descriptionTextField;

    @FXML
    private TextField nameTextField;

    @FXML
    private ComboBox<String> pollIntervalCombo;
    
    @FXML
    private TableView<WebFeedURL> urlTableView;
    
    @FXML
    private TableColumn<WebFeedURL, String> columnDescription;

    @FXML
    private TableColumn<WebFeedURL, String> columnName;

    @FXML
    private TableColumn<WebFeedURL, String> columnUrl;
    
    private boolean isNewMode = true;
    
    private WebFeedSubscription webFeedSubscription;
    
    
	@Override
	public void initialize(URL arg0, ResourceBundle arg1) {
		columnName.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("name"));
		columnUrl.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("urlText"));
		columnDescription.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("description"));
	}
	
	/**
	 * 
	 * @param wfs
	 * @param table
	 * @param newMode
	 */
	public void initialize(WebFeedSubscription wfs, 
			TableView<WebFeedSubscription> table, boolean newMode) {
		columnName.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("name"));
		columnUrl.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("urlText"));
		columnDescription.setCellValueFactory(
				new PropertyValueFactory<WebFeedURL, String>("description"));
		if(newMode) {
    		this.webFeedSubscription = wfs;
    	} else {
    		this.webFeedSubscription = wfs;
    		this.nameTextField.setText(wfs.getName());
    		this.nameTextField.setEditable(false);;
    		this.descriptionTextField.setText(wfs.getDescription());
        	this.pollIntervalCombo.getSelectionModel().select(
        			Interval.getValueOf(wfs.getPollInterval()).toString());
        	this.urlTableView.getItems().addAll(wfs.getUrls());
        	this.isNewMode = false;
    	}			
	}

	/**
	 * 
	 * @param event
	 */
    @FXML
    private void accionAcceptButton(ActionEvent event) {
    	String name = this.nameTextField.getText();
    	String description = this.descriptionTextField.getText();
    	String pi = (String) (this.pollIntervalCombo.getValue());
    	List<WebFeedURL> wfus = new ArrayList<WebFeedURL>();
    	for(WebFeedURL wfu: this.urlTableView.getItems()) {
    		wfus.add(wfu);
    	}    	 
    	
    	if(name.isEmpty() || description.isEmpty() || pi.isEmpty() || wfus.isEmpty()) {
    		new MessageDialog(new GuiException(GuiException.FORM_NOT_COMPLETE));
    	} else {
    		WebFeedSubscription wfs = null;
			
    		if(this.isNewMode) {
    			try {
    				wfs = new WebFeedSubscription( 
    						name, 
    						description,
    						wfus,
    						Interval.getValueOf(pi).getMilliseconds());
        			WebFeedSubscriptionInvoke.newWebFeedSubscription(wfs);
        		} catch (NamingException e) {
        			new MessageDialog(e);
        		}
    		} else {
    			try {
    				this.webFeedSubscription.setName(name);
    				this.webFeedSubscription.setDescription(description);
    				this.webFeedSubscription.setPollInterval(
    						Interval.getValueOf(pi).getMilliseconds());
    				this.webFeedSubscription.setUrls(wfus);;
        			WebFeedSubscriptionInvoke.editWebFeedSubscription(this.webFeedSubscription);
        		} catch (NamingException e) {
        			new MessageDialog(e);
        		}
    		} 		
    		
    		((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionCancelButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionAddURLButton(ActionEvent event) {
    	Stage stage = new Stage();
    	stage.setTitle("Select Web feed");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webfeed/SelectWebFeedURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
		SelectWebFeedURLViewController controller = 
				fxmlLoader.<SelectWebFeedURLViewController>getController();
    	controller.initialize(this.urlTableView);
    	Scene scene = new Scene(root, 600, 400, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
    /**
     * 
     * @param event
     */
    @FXML
    private void actionremoveURLButton(ActionEvent event) {
    	this.urlTableView.getItems().remove(
    			this.urlTableView.getSelectionModel().getSelectedItem());
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionNewURLButton(ActionEvent event) {
    	WebFeedURL wfu = null;
		try {
			wfu = new WebFeedURL();
		} catch (MalformedURLException e) {
			new MessageDialog(e);
		}
    	Stage stage = new Stage();
    	stage.setTitle("New Web feed URL");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webfeed/WebFeedURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
    	WebFeedURLViewController controller = fxmlLoader.<WebFeedURLViewController>getController();
    	if(wfu != null) {
    		controller.initialize(wfu, urlTableView, true);
    	}
    	Scene scene = new Scene(root, 400, 300, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
    /**
     * 
     * @param event
     */
    @FXML
    private void actionEditURLButton(ActionEvent event) {
    	WebFeedURL wfu = this.urlTableView.getSelectionModel().getSelectedItem();
    	Stage stage = new Stage();
    	stage.setTitle("Edit Web feed URL");
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource(
    			"/uoc/tfg/gui/views/webfeed/WebFeedURLView.fxml"));
    	Parent root = null;
		try {
			root = (Parent)fxmlLoader.load();
		} catch (IOException e) {
			new MessageDialog(e);
		}          
    	WebFeedURLViewController controller = 
    			fxmlLoader.<WebFeedURLViewController>getController();
    	controller.initialize(wfu, this.urlTableView, false);
    	Scene scene = new Scene(root, 400, 300, Color.WHITE);    
    	stage.setScene(scene);
    	stage.setResizable(false);
    	stage.initModality(Modality.APPLICATION_MODAL);
    	stage.show();
    }
    
}
