/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.gui.views.webfeed;

import java.net.URL;
import java.util.Date;
import java.util.ResourceBundle;

import uoc.tfg.exception.GuiException;
import uoc.tfg.gui.views.MessageDialog;
import uoc.tfg.invoke.webfeed.WebFeedURLInvoke;
import uoc.tfg.model.news.webfeed.WebFeedFormat;
import uoc.tfg.model.news.webfeed.WebFeedURL;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.stage.Stage;


public class WebFeedURLViewController implements Initializable {

    @FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Button acceptWebFeedURLButton;

    @FXML
    private Button cancelWebFeedURLButton;

    @FXML
    private TextArea descriptionWebFeedURLTextArea;

    @FXML
    private ComboBox<?> formatWebFeedURLComboBox;

    @FXML
    private TextField nameWebFeedURLTextField;

    @FXML
    private TextField urlWebFeedURLTextField;
    
    private Date lastPoll;
    
    private boolean isNewMode = true;
    
    private WebFeedURL webFeedURL;
    
    private TableView<WebFeedURL> tableView;
	


    @Override
    public void initialize(URL url, ResourceBundle rb) {
    }
    
    /**
     * 
     * @param wfu
     * @param tv
     * @param newMode
     */
    public void initialize(WebFeedURL wfu, TableView<WebFeedURL> tv, boolean newMode) {
    	this.isNewMode = newMode;
    	if(this.isNewMode) {
    		this.webFeedURL = wfu;
    		this.tableView = tv;
    	} else {
    		this.webFeedURL = wfu;
    		this.urlWebFeedURLTextField.setText(wfu.getUrl().toString());
    		this.urlWebFeedURLTextField.setEditable(false);
        	this.nameWebFeedURLTextField.setText(wfu.getName());
        	this.descriptionWebFeedURLTextArea.setText(wfu.getDescription());
        	this.formatWebFeedURLComboBox.getSelectionModel().select(wfu.getWebFeedformat().getValue());
        	this.lastPoll = wfu.getLastPoll();
    	}	
    }
    

    /**
     * 
     * @param event
     */
    @FXML
    private void accionAcceptWebFeedURLButton(ActionEvent event) {
    	String url = this.urlWebFeedURLTextField.getText();
    	String name = this.nameWebFeedURLTextField.getText();
    	String description = this.descriptionWebFeedURLTextArea.getText();
    	String wff = (String) (this.formatWebFeedURLComboBox.getValue());
    	
    	if(url.isEmpty() || name.isEmpty() || description.isEmpty() || wff.isEmpty()) {
    		new MessageDialog(new GuiException(GuiException.FORM_NOT_COMPLETE));
    	} else {
    		WebFeedURL wfu = null;
			
    		if(this.isNewMode) {
    			try {
    				wfu = new WebFeedURL(
    						url, 
    						name, 
    						description, 
    						WebFeedFormat.getValueOf(wff), 
    						new Date(0));
        			WebFeedURLInvoke.newWebFeedURL(wfu);
        		} catch (Exception e) {
        			new MessageDialog(e);
        		}
    			this.webFeedURL.setUrl(wfu.getUrl());
        		this.webFeedURL.setName(wfu.getName());
        		this.webFeedURL.setDescription(wfu.getDescription());
        		this.webFeedURL.setWebFeedformat(wfu.getWebFeedformat());
        		this.webFeedURL.setLastPoll(wfu.getLastPoll());
        		this.tableView.getItems().add(this.webFeedURL);
    		} else {
    			try {
    				this.webFeedURL.setName(name);
    				this.webFeedURL.setDescription(description);
    				this.webFeedURL.setWebFeedformat(WebFeedFormat.getValueOf(wff));
    				this.webFeedURL.setLastPoll(this.lastPoll);
        			WebFeedURLInvoke.editWebFeedURL(this.webFeedURL);
        		} catch (Exception e) {
        			new MessageDialog(e);
        		}
    		}
    		   		
        	
    		((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    	}
    }

    /**
     * 
     * @param event
     */
    @FXML
    private void actionCancelWebFeedURLButton(ActionEvent event) {
    	((Stage) ((Node)  event.getSource()).getScene().getWindow()).close();
    }
    
}
