/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 12/12/2013
 *
 */
package uoc.tfg.invoke.webcrawling;

import java.io.IOException;
import java.io.Serializable;
import java.util.Observable;
import java.util.Observer;

import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import javax.naming.NamingException;

import uoc.tfg.exception.WebCrawlingException;
import uoc.tfg.gui.views.webcrawling.WebCrawlerModel;
import uoc.tfg.gui.views.webcrawling.WebCrawlerViewController;
import uoc.tfg.gui.views.webcrawling.WebCrawlingEntryTableModel;
import uoc.tfg.manager.webcrawling.IWebCrawlerManager;
import uoc.tfg.manager.webcrawling.WebCrawlerManager;
import uoc.tfg.model.document.Index;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;

/*
 *
 */
public class WebCrawlerInvoke implements Observer, Serializable {

	private static final long serialVersionUID = -2618096929898705638L;
	private IWebCrawlerManager manager;
	private WebCrawlerModel crawlerModel = new WebCrawlerModel();
	private ObservableList<WebCrawlingEntryTableModel> entries = 
			FXCollections.observableArrayList();
	private WebCrawlerViewController wcvc;
		
	/**
	 * 
	 * @param wfs
	 * @throws IOException 
	 * @throws NamingException
	 */
	public WebCrawlerInvoke(WebCrawlerViewController wcvc) throws IOException {
		
		this.manager = new WebCrawlerManager(wcvc.getWebCrawlingStrategy(), wcvc.getIndex(), this);
		this.wcvc = wcvc;
		this.wcvc.getStatusText().setText(this.manager.getStatus());
		this.wcvc.getVisitedUrlsText().setText("0");
	}
	
	/**
	 * 
	 * @throws WebFeedException
	 * @throws WebCrawlingException 
	 */
	public void startWebCrawler() throws WebCrawlingException {
		this.manager.startWebCrawler();
	}

	/**
	 * 
	 */
	public void pausetWebCrawler() {
		this.manager.pauseWebCrawler();
	}

	/**
	 * 
	 */
	public void resumeWebCrawler() {
		this.manager.pauseWebCrawler();
	}

	/**
	 * 
	 */
	public void stopWebCrawler() {
		this.manager.stopWebCrawler();
	}
	
	/**
	 * 
	 * @return
	 */
	public StringProperty getStatusProperty() {
		return this.crawlerModel.statusProperty();
	}
	
	/**
	 * 
	 * @return
	 */
	public StringProperty getVisitedUrlsProperty() {
		return this.crawlerModel.visitedUrlsProperty();
	}

	/**
	 * 
	 */
	@Override
	public void update(Observable o, Object arg) {
		
		this.wcvc.getStatusText().setText(this.manager.getStatus());
		this.wcvc.getVisitedUrlsText().setText(String.valueOf(this.manager.getVisitedUrls()));
		while(!((IWebCrawlerManager)o).getLastEntries().isEmpty()) {
			this.entries.add(0, new WebCrawlingEntryTableModel(((IWebCrawlerManager)o).getLastEntries().poll()));
		}
	}

	/**
	 * 
	 * @return
	 */
	public ObservableList<WebCrawlingEntryTableModel> getEntries() {
		return entries;
	}

	/**
	 * 
	 * @param entries
	 */
	public void setEntries(ObservableList<WebCrawlingEntryTableModel> entries) {
		this.entries = entries;
	}

}
