/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.invoke.webfeed;

import java.io.IOException;
import java.io.Serializable;
import java.util.Observable;
import java.util.Observer;

import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import javax.naming.NamingException;

import uoc.tfg.exception.WebFeedException;
import uoc.tfg.gui.views.webfeed.WebFeedEntryTableModel;
import uoc.tfg.gui.views.webfeed.WebFeedSubscriptionTableModel;
import uoc.tfg.manager.webfeed.IWebFeedReaderManager;
import uoc.tfg.manager.webfeed.WebFeedReaderManager;
import uoc.tfg.model.document.Index;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;


public class WebFeedReaderInvoke implements Observer, Serializable {

	private static final long serialVersionUID = 8793192526080092007L;
	private IWebFeedReaderManager manager;
	private WebFeedSubscriptionTableModel subscription = new WebFeedSubscriptionTableModel();
	private ObservableList<WebFeedEntryTableModel> entries = 
			FXCollections.observableArrayList();
		
	/**
	 * 
	 * @param wfs
	 * @throws NamingException
	 * @throws IOException 
	 */
	public WebFeedReaderInvoke(WebFeedSubscription wfs, Index index) throws NamingException, IOException {
		this.manager = new WebFeedReaderManager(wfs, index, this);
		this.subscription.setName(this.manager.getName());
		this.subscription.setStatus(this.manager.getStatus());
	}
	
	public void setIndex(Index i) throws IOException {
		this.manager.setIndex(i);
	}
	
	/**
	 * 
	 * @throws WebFeedException
	 */
	public void startWebFeedReader() throws WebFeedException {
		this.manager.startWebFeedReader();
	}

	/**
	 * 
	 */
	public void pausetWebFeedReader() {
		this.manager.pausetWebFeedReader();
	}

	/**
	 * 
	 */
	public void resumeWebFeedReader() {
		this.manager.resumeWebFeedReader();
	}

	/**
	 * 
	 */
	public void stopWebFeedReader() {
		this.manager.stopWebFeedReader();
	}
	
	/**
	 * 
	 * @return
	 */
	public StringProperty getStatusProperty() {
		return this.subscription.statusProperty();
	}
	
	/**
	 * 
	 * @return
	 */
	public StringProperty getNameProperty() {
		return this.subscription.nameProperty();
	}

	/**
	 * 
	 */
	@Override
	public void update(Observable o, Object arg) {	
		this.subscription.setStatus(((IWebFeedReaderManager)o).getStatus());
		while(!((IWebFeedReaderManager)o).getLastEntries().isEmpty()) {
			this.entries.add(0,new WebFeedEntryTableModel(((IWebFeedReaderManager)o).getLastEntries().poll()));
		}
	}

	/**
	 * 
	 * @return
	 */
	public ObservableList<WebFeedEntryTableModel> getEntries() {
		return entries;
	}

	/**
	 * 
	 * @param entries
	 */
	public void setEntries(ObservableList<WebFeedEntryTableModel> entries) {
		this.entries = entries;
	}

	
}
