package uoc.tfg.jpa.analysis;

import java.io.Serializable;

import javax.persistence.*;

import uoc.tfg.model.textanalysis.classification.Category;
import uoc.tfg.model.textanalysis.classification.CategorySet;

import java.util.ArrayList;
import java.util.List;


/**
 * The persistent class for the category_set database table.
 * 
 */
@Entity
@Table(name="category_set")
@NamedQuery(name="CategorySetJPA.findAll", query="SELECT c FROM CategorySetJPA c")
public class CategorySetJPA implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	private String set_name;

	private String set_description;

	//uni-directional many-to-many association to CategoryJPA
	@ManyToMany(cascade= CascadeType.ALL)
	@JoinTable(
		name="categories_in_set"
		, joinColumns={
			@JoinColumn(name="set_name")
			}
		, inverseJoinColumns={
			@JoinColumn(name="category_name", referencedColumnName="category_name"),
			@JoinColumn(name="category_value", referencedColumnName="category_value")
			}
		)
	private List<CategoryJPA> categories = new ArrayList<CategoryJPA>();

	public CategorySetJPA() {
	}

	public CategorySetJPA(CategorySet cs) {

		this.set_name = cs.getName();
		this.set_description = cs.getDescription();
		for(Category c: cs.getCategories()) {
			this.categories.add(new CategoryJPA(c));
		}
	}
	
	public CategorySet getCategorySet() {
		
		Category[] cs = new Category[this.categories.size()];
		for(CategoryJPA c: this.categories) {
			cs[c.getId().getCategory_value()] = c.getCategory();
		}
		CategorySet categorySet = new CategorySet(this.set_name, this.set_description, cs);
		
		return categorySet;
	}

	public String getSet_name() {
		return this.set_name;
	}

	public void setSet_name(String set_name) {
		this.set_name = set_name;
	}

	public String getSet_description() {
		return this.set_description;
	}

	public void setSet_description(String set_description) {
		this.set_description = set_description;
	}

	public List<CategoryJPA> getCategories() {
		return this.categories;
	}

	public void setCategories(List<CategoryJPA> categories) {
		this.categories = categories;
	}

}