package uoc.tfg.jpa.analysis;

import java.io.Serializable;

import javax.persistence.*;

import uoc.tfg.model.textanalysis.IToken;
import uoc.tfg.model.textanalysis.ITokenCollection;
import uoc.tfg.model.textanalysis.TokenCollection;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * The persistent class for the token_collection database table.
 * 
 */
@Entity
@Table(name="token_collection")
@NamedQuery(name="TokenCollectionJPA.findAll", query="SELECT t FROM TokenCollectionJPA t")
public class TokenCollectionJPA implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	private String collection_name;

	private String collection_description;

	//bi-directional many-to-one association to TokenJPA
	@OneToMany(mappedBy="tokenCollection", cascade= CascadeType.ALL)
	private List<TokenJPA> tokens = new ArrayList<TokenJPA>();

	public TokenCollectionJPA() {
	}
	
	public TokenCollectionJPA(ITokenCollection tc) {
		
		this.setCollection_name(tc.getName());
		this.setCollection_description(tc.getDescription());
		for(IToken t: tc.getTokens()) {
			this.tokens.add(new TokenJPA(t, this));
		}
	}
	
	public ITokenCollection getTokenCollection() {
		
		Set<String> tks = new HashSet<String>();
		for(TokenJPA t: this.tokens) {
			tks.add(t.getId().getToken());
		}
		
		ITokenCollection tc = new TokenCollection(tks);
		tc.setName(this.getCollection_name());
		tc.setDescription(this.getCollection_description());
		
		return tc;
	}

	public String getCollection_name() {
		return this.collection_name;
	}

	public void setCollection_name(String collection_name) {
		this.collection_name = collection_name;
	}

	public String getCollection_description() {
		return this.collection_description;
	}

	public void setCollection_description(String collection_description) {
		this.collection_description = collection_description;
	}

	public List<TokenJPA> getTokens() {
		return this.tokens;
	}

	public void setTokens(List<TokenJPA> tokens) {
		this.tokens = tokens;
	}

	public TokenJPA addToken(TokenJPA token) {
		getTokens().add(token);
		token.setTokenCollection(this);

		return token;
	}

	public TokenJPA removeToken(TokenJPA token) {
		getTokens().remove(token);
		token.setTokenCollection(null);

		return token;
	}

}