package uoc.tfg.jpa.news;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;

import javax.persistence.*;

import uoc.tfg.model.news.WebContent;
import uoc.tfg.model.news.WebContentFormat;

import java.util.Date;


/**
 * The persistent class for the web_content database table.
 * 
 */
@Entity
@Table(name="web_content")
@NamedQuery(name="WebContentJPA.findAll", query="SELECT w FROM WebContentJPA w")
public class WebContentJPA implements Serializable {

	private static final long serialVersionUID = -1880768606113791259L;

	@Id
	private String url;

	private String content;

	private String format;

	@Temporal(TemporalType.TIMESTAMP)
	private Date last_modified;

	private String title;
	
	public WebContentJPA() {
		super();
	}

	public WebContentJPA(WebContent wc) {
		this.url = wc.getUrl().toString();
		this.title = wc.getTitle();
		this.content = wc.getContent();
		this.format = wc.getFormat().toString();
		this.last_modified = wc.getLastModified();
	}
	
	public WebContent getWebContent() throws MalformedURLException {
		return new WebContent(
				new URL(this.url),
				this.title,
				WebContentFormat.getValueOf(this.format),
				this.last_modified,
				this.content);
	}

	public String getUrl() {
		return this.url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getContent() {
		return this.content;
	}

	public void setContent(String content) {
		this.content = content;
	}

	public String getFormat() {
		return this.format;
	}

	public void setFormat(String format) {
		this.format = format;
	}

	public Date getLastModified() {
		return this.last_modified;
	}

	public void setLastModified(Date lastModified) {
		this.last_modified = lastModified;
	}

	public String getTitle() {
		return this.title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

}