package uoc.tfg.jpa.news.webcrawling;

import java.io.Serializable;
import java.net.MalformedURLException;

import javax.persistence.*;

import uoc.tfg.model.news.webcrawler.WebCrawlingSeed;
import uoc.tfg.model.news.webcrawler.WebCrawlingURL;

import java.util.ArrayList;
import java.util.List;


/**
 * The persistent class for the crawling_seed database table.
 * 
 */
@Entity
@Table(name="crawling_seed")
@NamedQuery(name="WebCrawlingSeedJPA.findAll", query="SELECT c FROM WebCrawlingSeedJPA c")
public class WebCrawlingSeedJPA implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@Column(name="seed_name")
	private String seedName;

	private String description;

	//uni-directional many-to-many association to CrawlingURLJPA
	@ManyToMany
	@JoinTable(
		name="crawling_seed_url"
		, joinColumns={
			@JoinColumn(name="seed")
			}
		, inverseJoinColumns={
			@JoinColumn(name="url")
			}
		)
	private List<WebCrawlingURLJPA> crawlingUrls = new ArrayList<WebCrawlingURLJPA>();

	
	public WebCrawlingSeedJPA() {
		super();
		// TODO Auto-generated constructor stub
	}

	public WebCrawlingSeedJPA(WebCrawlingSeed wcs) {
		this.seedName = wcs.getName();
		this.description = wcs.getDescription();
		for(WebCrawlingURL wcu: wcs.getUrls()) {
			this.crawlingUrls.add(new WebCrawlingURLJPA(wcu));
		}
	}
	
	public WebCrawlingSeed getWebCrawlingSeed() throws MalformedURLException {
		List<WebCrawlingURL> wcus = new ArrayList<WebCrawlingURL>();
		for(WebCrawlingURLJPA wcu: this.getCrawlingUrls()) {
			wcus.add(wcu.getWebCrawlingURL());
		}
		return new WebCrawlingSeed(this.seedName, this.description, wcus);
	}

	public String getSeedName() {
		return this.seedName;
	}

	public void setSeedName(String seedName) {
		this.seedName = seedName;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<WebCrawlingURLJPA> getCrawlingUrls() {
		return this.crawlingUrls;
	}

	public void setCrawlingUrls(List<WebCrawlingURLJPA> crawlingUrls) {
		this.crawlingUrls = crawlingUrls;
	}
	
	public void setCrawlingURLs(List<WebCrawlingURL> crawlingUrls) {
		for(WebCrawlingURL wcu: crawlingUrls) {
			this.crawlingUrls.add(new WebCrawlingURLJPA(wcu));
		}
	}

}