package uoc.tfg.jpa.news.webcrawling;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.*;

import uoc.tfg.jpa.news.webfeed.WebFeedURLJPA;
import uoc.tfg.model.news.WebContentSimpleParser;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;
import uoc.tfg.model.news.webcrawler.WebCrawlingSimpleStrategy;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;
import uoc.tfg.model.news.webfeed.WebFeedURL;

/**
 * The persistent class for the crawling_strategy database table.
 * 
 */
@Entity
@Table(name="crawling_strategy")
@NamedQuery(name="WebCrawlingStrategyJPA.findAll", query="SELECT c FROM WebCrawlingStrategyJPA c")
public class WebCrawlingStrategyJPA implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@Column(name="strategy_name")
	private String strategyName;

	private String algorithm;

	private Long delay;

	private String description;

	@Column(name="max_depth")
	private Integer maxDepth;

	@Column(name="max_num_url")
	private Integer maxNumUrl;

	//uni-directional many-to-one association to CrawlingSeed
	@ManyToOne
	@JoinColumn(name="seed")
	private WebCrawlingSeedJPA crawlingSeed = new WebCrawlingSeedJPA();
	
	public WebCrawlingStrategyJPA() {
		super();
	}

	public WebCrawlingStrategyJPA(WebCrawlingSimpleStrategy wcst) {
		
		this.strategyName = wcst.getStrategyName();
		this.description = wcst.getDescription();
		this.maxNumUrl = wcst.getMaxNumUrl();
		this.maxDepth = wcst.getMaxDepth();
		this.algorithm = wcst.getAlgorithm();
		this.delay = wcst.getDelay();
		this.crawlingSeed = new WebCrawlingSeedJPA(wcst.getSeed());
	}
	
	public IWebCrawlingStrategy getWebCrawlingStrategy() throws MalformedURLException {
		
		IWebCrawlingStrategy wcs = new WebCrawlingSimpleStrategy(
				this.getStrategyName(), 
				this.getDescription(), 
				this.maxNumUrl.intValue(), 
				this.getMaxDepth().intValue(), 
				this.getAlgorithm(), 
				this.getDelay().longValue(), 
				this.getCrawlingSeed().getWebCrawlingSeed(), 
				new WebContentSimpleParser());
		return wcs;
	}

	public String getStrategyName() {
		return this.strategyName;
	}

	public void setStrategyName(String strategyName) {
		this.strategyName = strategyName;
	}

	public String getAlgorithm() {
		return this.algorithm;
	}

	public void setAlgorithm(String algorithm) {
		this.algorithm = algorithm;
	}

	public Long getDelay() {
		return this.delay;
	}

	public void setDelay(Long delay) {
		this.delay = delay;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Integer getMaxDepth() {
		return this.maxDepth;
	}

	public void setMaxDepth(Integer maxDepth) {
		this.maxDepth = maxDepth;
	}

	public Integer getMaxNumUrl() {
		return this.maxNumUrl;
	}

	public void setMaxNumUrl(Integer maxNumUrl) {
		this.maxNumUrl = maxNumUrl;
	}

	public WebCrawlingSeedJPA getCrawlingSeed() {
		return this.crawlingSeed;
	}

	public void setCrawlingSeed(WebCrawlingSeedJPA crawlingSeed) {
		this.crawlingSeed = crawlingSeed;
	}

}