/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.jpa.news.webfeed;

import java.io.Serializable;
import java.net.MalformedURLException;

import javax.persistence.*;

import uoc.tfg.model.news.webfeed.WebFeedSubscription;
import uoc.tfg.model.news.webfeed.WebFeedURL;

import java.util.ArrayList;
import java.util.List;


@Entity
@Table(name="feed_subscription")
@NamedQuery(name="WebFeedSubscriptionJPA.findAll", query="SELECT w FROM WebFeedSubscriptionJPA w")
public class WebFeedSubscriptionJPA implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@Column(name="subscription_name")
	private String subscriptionName;
	
	@Column(name="subscription_description")
	private String subscriptionDescription;

	private long pollinterval;

	//uni-directional many-to-many association to WebFeedURLJPA
	@ManyToMany
	@JoinTable(
		name="feed_subscription_url"
		, joinColumns={
			@JoinColumn(name="subscription")
			}
		, inverseJoinColumns={
			@JoinColumn(name="url")
			}
		)
	private List<WebFeedURLJPA> feedUrls = new ArrayList<WebFeedURLJPA>();

	/**
	 * 
	 * Constructor
	 *
	 * WebFeedSubscriptionJPA
	 */
	public WebFeedSubscriptionJPA() {
	}
	
	/**
	 * 
	 * Constructor
	 *
	 * WebFeedSubscriptionJPA
	 */
	public WebFeedSubscriptionJPA(WebFeedSubscription wfs) {
		this.subscriptionName = wfs.getName();
		this.subscriptionDescription = wfs.getDescription();
		this.setUrls(wfs.getUrls());
		this.pollinterval = wfs.getPollInterval();
	}

	/**
	 * 
	 * Constructor
	 *
	 * WebFeedSubscriptionJPA
	 */
	public WebFeedSubscriptionJPA(String name, String description,
			List<WebFeedURLJPA> urls, long pollInterval) {
		this.subscriptionName = name;
		this.subscriptionDescription = description;
		this.feedUrls = urls;
		this.pollinterval = pollInterval;
	}
	
	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return void
	 */
	public void setUrls(List<WebFeedURL> urls) {
		this.feedUrls.clear();
		for(WebFeedURL url: urls) {
			this.feedUrls.add(new WebFeedURLJPA(url));
		}
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return WebFeedSubscription
	 */
	public WebFeedSubscription getWebFeedSubscription() throws MalformedURLException {
		
		List<WebFeedURL> newUrls = new ArrayList<WebFeedURL>();
		for(WebFeedURLJPA url: this.feedUrls) {
			newUrls.add(url.getWebFeedURL());
		}
		WebFeedSubscription wfs = new WebFeedSubscription(
				this.getSubscriptionName(), 
				this.getSubscriptionDescription(), 
				newUrls, 
				this.getPollinterval());
		return wfs;
		
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return String
	 */
	public String getSubscriptionName() {
		return this.subscriptionName;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return void
	 */
	public void setSubscriptionName(String subscriptionName) {
		this.subscriptionName = subscriptionName;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return long
	 */
	public long getPollinterval() {
		return this.pollinterval;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return void
	 */
	public void setPollinterval(long pollinterval) {
		this.pollinterval = pollinterval;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return String
	 */
	public String getSubscriptionDescription() {
		return this.subscriptionDescription;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return void
	 */
	public void setSubscriptionDescription(String subscriptionDescription) {
		this.subscriptionDescription = subscriptionDescription;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return List<WebFeedURLJPA>
	 */
	public List<WebFeedURLJPA> getFeedUrls() {
		return this.feedUrls;
	}

	/**
	 * 
	 * @param WebFeedSubscriptionJPA
	 * @return void
	 */
	public void setFeedUrls(List<WebFeedURLJPA> feedUrls) {
		this.feedUrls = feedUrls;
	}

}