package uoc.tfg.manager.document;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.document.IndexJPA;
import uoc.tfg.model.document.Index;

public class IndexManager implements IIndexManager {

	@Override
	public void newIndex(Index i) {

		DatabaseManager.persist(new IndexJPA(i));
	}

	@Override
	public void editIndex(Index i) {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		IndexJPA ijpa = entitymanager.find(IndexJPA.class, i.getPath().toString());
		ijpa.setName(i.getName());
		ijpa.setDescription(i.getDescription());
    	entitymanager.getTransaction().commit();
    	entitymanager.close();
	}

	@Override
	public void refreshIndex(Index i) {

		DatabaseManager.refresh(new IndexJPA(i));
	}

	@Override
	public void deleteIndex(String path) throws DatabaseException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		IndexJPA ijpa = entitymanager.find(IndexJPA.class, path);
		if(ijpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			entitymanager.remove(ijpa);
		}
		entitymanager.getTransaction().commit();
		entitymanager.close();	
	}

	@Override
	public Index getIndex(String path) throws DatabaseException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		IndexJPA ijpa = entitymanager.find(IndexJPA.class, path);
		if(ijpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			return ijpa.getIndex();
		}
	}

	@Override
	public List<Index> getAllIndex() {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		List<Index> idxs = new ArrayList<Index>();
		Query query = entitymanager.createQuery("SELECT e FROM IndexJPA e");
		List<IndexJPA> ijpas = (List<IndexJPA>) query.getResultList();
		for(IndexJPA ijpa: ijpas) {
			idxs.add(ijpa.getIndex());
		}
		return idxs;
	}

}
