/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 17/12/2013
 *
 */
package uoc.tfg.manager.marketdata;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import com.lightstreamer.ls_client.UpdateInfo;

import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.marketdata.LightstreamerJPA;
import uoc.tfg.jpa.marketdata.UnitaryTickDataJPA;
import uoc.tfg.jpa.marketdata.UnitaryTickDataPK;
import uoc.tfg.jpa.news.webfeed.WebFeedURLJPA;
import uoc.tfg.model.marketdata.UnitaryTickData;
import uoc.tfg.model.marketdata.lightstreamer.AdapterInfoLS;
import uoc.tfg.model.marketdata.lightstreamer.DataFeederLS;
import uoc.tfg.model.marketdata.lightstreamer.IDataPullerLS;
import uoc.tfg.model.news.webfeed.WebFeedURL;


public class MarketDataLSManager extends Observable implements IDataPullerLS, IMarketDataLSManager {
	
	private final SimpleDateFormat sdf1 = new SimpleDateFormat("dd/MM/yyHH:mm:ss");
	private final SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private DataFeederLS datafeeder;
	private AdapterInfoLS info = new AdapterInfoLS();
	private UpdateInfo lastUpdate;
	private int lastUpdatePosition;
	
	public MarketDataLSManager() {
		this.datafeeder = new DataFeederLS(info, this);
	}
	
	

	@Override
	public void startMarketDataStreaming() throws Exception {
		this.datafeeder.start();
	}
	
	@Override
	public void pauseMarketDataStreaming() throws Exception {
		this.datafeeder.pause();
	}
	
	@Override
	public void resumeMarketDataStreaming() throws Exception {
		this.datafeeder.resume();
	}
	
	@Override
	public void stopMarketDataStreaming() throws Exception {
		this.datafeeder.stop();
	}
	

	/* (non-Javadoc)
	 * @see model.marketdata.lightstreamer.IDataPullerLS#update(model.marketdata.UnitaryTickData)
	 */
	@Override
	public synchronized void update(UnitaryTickData update, UpdateInfo updateInfo) {
		
		UnitaryTickDataJPA updateJPA = new UnitaryTickDataJPA();
		UnitaryTickDataPK updatePK = new UnitaryTickDataPK();
		updatePK.setReference(update.getReference());
		updatePK.setVolume(update.getVolume());
		updateJPA.setId(updatePK);
		updateJPA.setPrice(update.getPrice());
		updateJPA.setTickTime(update.getTickTime());
		
		try {
			DatabaseManager.persist(updateJPA);
		} catch(Exception e) {}
		
		this.setLastUpdate(updateInfo);
	}
	
	@Override
	public AdapterInfoLS getInfo() {
		return info;
	}

	@Override
	public void setInfo(AdapterInfoLS info) {
		this.info = info;
	}
	
	@Override
	public void setObserver(Observer o) {
		this.addObserver(o);
	}

	@Override
	public UpdateInfo getLastUpdate() {
		return lastUpdate;
	}



	public void setLastUpdate(UpdateInfo lastUpdate) {
		this.lastUpdate = lastUpdate;
		this.setLastUpdatePosition(Arrays.asList(this.info.getItems()).indexOf(lastUpdate.getItemName()));
		this.setChanged();
		this.notifyObservers();
	}


	@Override
	public int getLastUpdatePosition() {
		return lastUpdatePosition;
	}

	public void setLastUpdatePosition(int lastUpdatePosition) {
		this.lastUpdatePosition = lastUpdatePosition;
	}



	@Override
	public List<LightstreamerJPA> getAllServer() {
		
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		
		Query query = entitymanager.createQuery("SELECT e FROM LightstreamerJPA e");
		List<LightstreamerJPA> sJPAs = (List<LightstreamerJPA>) query.getResultList();
		
		return sJPAs;
	}

}
