/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 12/12/2013
 *
 */
package uoc.tfg.manager.webcrawling;

import java.io.IOException;
import java.util.Observable;
import java.util.Observer;
import java.util.Queue;

import org.apache.lucene.document.Document;

import uoc.tfg.exception.WebCrawlingException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.WebContentJPA;
import uoc.tfg.model.Status;
import uoc.tfg.model.document.IIndexHandler;
import uoc.tfg.model.document.Index;
import uoc.tfg.model.document.IndexHandler;
import uoc.tfg.model.news.IWebContentPuller;
import uoc.tfg.model.news.WebContent;
import uoc.tfg.model.news.webcrawler.IWebCrawler;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;
import uoc.tfg.model.news.webcrawler.WebCrawler;




public class WebCrawlerManager extends Observable implements IWebCrawlerManager, IWebContentPuller {
	
	private IWebCrawler webCrawler;
	private IIndexHandler indexManager;
	
	public WebCrawlerManager(IWebCrawlingStrategy wcs, Index index, Observer o) throws IOException {
		this.webCrawler = new WebCrawler(wcs, this);
		this.indexManager = new IndexHandler(index.getPath());
		this.addObserver(o);
	}

	
	@Override
	public String getName() {
		if(this.webCrawler != null)
			return this.webCrawler.getName();
		else return null;
	}

	@Override
	public String getStatus() {
		if(this.webCrawler != null)
			return this.webCrawler.getStatus().toString();
		return null;
	}
	
	@Override
	public int getVisitedUrls() {
		if(this.webCrawler != null)
			return this.webCrawler.getNumVisitedUrls();
		return -1;
	}

	@Override
	public void setStatus(Status status) {
		if(this.webCrawler != null)
			this.webCrawler.setStatus(status);
	}

	@Override
	public void startWebCrawler() throws WebCrawlingException {
		if(this.webCrawler != null)
			this.webCrawler.startWebCrawler();
	}

	@Override
	public void pauseWebCrawler() {
		if(this.webCrawler != null)
			this.webCrawler.pauseWebCrawler();
	}

	@Override
	public void resumeWebCrawler() {
		if(this.webCrawler != null)
			this.webCrawler.resumeWebCrawler();
	}

	@Override
	public void stopWebCrawler() {
		if(this.webCrawler != null)
			this.webCrawler.stopWebCrawler();
	}

	@Override
	public void update(WebContent content) throws Exception {
		WebContentJPA cJPA = new WebContentJPA(content);
		if(content != null) {
			DatabaseManager.persist(cJPA);
		}
	}
	
	@Override
	public void update(Document doc) {
		
		try {
			indexManager.createIndexWriter();
			indexManager.addDocument(doc);
		} catch (IOException e) {}
	}
	
	@Override
	public void update(Observable arg0, Object arg1) {
		this.setChanged();
		this.notifyObservers();
	}

	@Override
	public Queue<WebContent> getLastEntries() {
		return this.webCrawler.getLastEntries();
	}


}
