/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 23/12/2013
 *
 */
package uoc.tfg.manager.webcrawling;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.webcrawling.WebCrawlingSeedJPA;
import uoc.tfg.jpa.news.webcrawling.WebCrawlingURLJPA;
import uoc.tfg.jpa.news.webfeed.WebFeedSubscriptionJPA;
import uoc.tfg.model.news.webcrawler.WebCrawlingSeed;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;

/*
 *
 */
public class WebCrawlingSeedManager implements IWebCrawlingSeedManager {

	
	@Override
	public void newWebCrawlingSeed(WebCrawlingSeed wcs) {

		DatabaseManager.persist(new WebCrawlingSeedJPA(wcs));
	}

	@Override
	public void refreshWebCrawlingSeed(WebCrawlingSeed wcs) {

		DatabaseManager.refresh(new WebCrawlingSeedJPA(wcs));
	}

	@Override
	public void editWebCrawlingSeed(WebCrawlingSeed wcs) {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingSeedJPA wcsjpa = entitymanager.find(WebCrawlingSeedJPA.class, wcs.getName());
		wcsjpa.setDescription(wcs.getDescription());
		wcsjpa.setCrawlingURLs(wcs.getURLs());
    	entitymanager.getTransaction().commit();
    	entitymanager.close();
	}

	
	@Override
	public void deleteWebCrawlingSeed(String name) throws DatabaseException {
		
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingSeedJPA wcsjpa = entitymanager.find(WebCrawlingSeedJPA.class, name);
		if(wcsjpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			entitymanager.remove(wcsjpa);
		}
		entitymanager.getTransaction().commit();
		entitymanager.close();		
	}

	
	@Override
	public WebCrawlingSeed getWebCrawlingSeed(String name) throws DatabaseException, MalformedURLException {
		
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		WebCrawlingSeedJPA wcsjpa = entitymanager.find(WebCrawlingSeedJPA.class, name);
		entitymanager.close();
		if(wcsjpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			return wcsjpa.getWebCrawlingSeed();
		}
	}

	
	@Override
	public List<WebCrawlingSeed> getAllWebCrawlingSeed() throws MalformedURLException {
		
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		List<WebCrawlingSeed> wfss = new ArrayList<WebCrawlingSeed>();
		Query query = entitymanager.createQuery("SELECT e FROM WebCrawlingSeedJPA e");
		List<WebCrawlingSeedJPA> wfsJPAs = (List<WebCrawlingSeedJPA>) query.getResultList();
		entitymanager.close();
		for(WebCrawlingSeedJPA wfsJPA: wfsJPAs) {
			wfss.add(wfsJPA.getWebCrawlingSeed());
		}
		return wfss;
	}

}
