/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 23/12/2013
 *
 */
package uoc.tfg.manager.webcrawling;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.webcrawling.WebCrawlingSeedJPA;
import uoc.tfg.jpa.news.webcrawling.WebCrawlingStrategyJPA;
import uoc.tfg.model.news.webcrawler.IWebCrawlingStrategy;
import uoc.tfg.model.news.webcrawler.WebCrawlingSimpleStrategy;

/*
 *
 */
public class WebCrawlingStrategyManager implements IWebCrawlingStrategyManager {

	
	@Override
	public void newWebCrawlingStrategy(IWebCrawlingStrategy wcs) {

		DatabaseManager.persist(new WebCrawlingStrategyJPA((WebCrawlingSimpleStrategy) wcs));
	}

	@Override
	public void refreshWebCrawlingStrategy(IWebCrawlingStrategy wcs) {
		
		DatabaseManager.refresh(new WebCrawlingStrategyJPA((WebCrawlingSimpleStrategy) wcs));
	}
	
	@Override
	public void editWebCrawlingStrategy(IWebCrawlingStrategy wcs) {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingStrategyJPA wcsjpa = entitymanager.find(WebCrawlingStrategyJPA.class, wcs.getStrategyName());
		wcsjpa.setDescription(wcs.getDescription());
		wcsjpa.setCrawlingSeed(new WebCrawlingSeedJPA(wcs.getSeed()));
		wcsjpa.setAlgorithm(wcs.getAlgorithm());
		wcsjpa.setMaxDepth(wcs.getMaxDepth());
		wcsjpa.setMaxNumUrl(wcs.getMaxNumUrl());
		wcsjpa.setDelay(wcs.getDelay());
    	entitymanager.getTransaction().commit();
    	entitymanager.close();
	}

	
	@Override
	public void deleteWebCrawlingStrategy(String name) throws DatabaseException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingStrategyJPA wcsjpa = entitymanager.find(WebCrawlingStrategyJPA.class, name);
		if(wcsjpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			entitymanager.remove(wcsjpa);
		}
		entitymanager.getTransaction().commit();
		entitymanager.close();	
	}

	
	@Override
	public IWebCrawlingStrategy getWebCrawlingStrategy(String name) throws DatabaseException, MalformedURLException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		WebCrawlingStrategyJPA wcsjpa = entitymanager.find(WebCrawlingStrategyJPA.class, name);
		entitymanager.close();
		if(wcsjpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			return wcsjpa.getWebCrawlingStrategy();
		}
	}

	
	@Override
	public List<IWebCrawlingStrategy> getAllWebCrawlingStrategy() throws MalformedURLException {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		List<IWebCrawlingStrategy> wcss = new ArrayList<IWebCrawlingStrategy>();
		Query query = entitymanager.createQuery("SELECT e FROM WebCrawlingStrategyJPA e");
		List<WebCrawlingStrategyJPA> wcsJPAs = (List<WebCrawlingStrategyJPA>) query.getResultList();
		entitymanager.close();
		for(WebCrawlingStrategyJPA wcsJPA: wcsJPAs) {
			wcss.add(wcsJPA.getWebCrawlingStrategy());
		}
		
		return wcss;
	}

}
