/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 23/12/2013
 *
 */
package uoc.tfg.manager.webcrawling;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.webcrawling.WebCrawlingURLJPA;
import uoc.tfg.model.news.webcrawler.WebCrawlingURL;


public class WebCrawlingURLManager implements IWebCrawlingURLManager {

	
	@Override
	public void newWebCrawlingURL(WebCrawlingURL wcu) {
		
		DatabaseManager.persist(new WebCrawlingURLJPA(wcu));
	}

	
	@Override
	public void refreshWebCrawlingURL(WebCrawlingURL wcu) {

		DatabaseManager.refresh(new WebCrawlingURLJPA(wcu));
	}
	
	@Override
	public void editWebCrawlingURL(WebCrawlingURL wcu) {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingURLJPA wcujpa = entitymanager.find(WebCrawlingURLJPA.class, wcu.getUrlText());
		wcujpa.setUrlName(wcu.getName());
		wcujpa.setDescription(wcu.getDescription());
    	entitymanager.getTransaction().commit();
    	entitymanager.close();
	}

	
	@Override
	public void deleteWebCrawlingURL(String url) throws DatabaseException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebCrawlingURLJPA wcujpa = entitymanager.find(WebCrawlingURLJPA.class, url);
		if(wcujpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			entitymanager.remove(wcujpa);
		}
		entitymanager.getTransaction().commit();
		entitymanager.close();	
		
	}

	
	@Override
	public WebCrawlingURL getWebCrawlingURL(String url) throws DatabaseException, MalformedURLException {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		WebCrawlingURLJPA wcujpa = entitymanager.find(WebCrawlingURLJPA.class, url);
		if(wcujpa == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			return wcujpa.getWebCrawlingURL();
		}
	}

	
	@Override
	public List<WebCrawlingURL> getAllWebCrawlingURL() throws MalformedURLException {

		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		List<WebCrawlingURL> wcus = new ArrayList<WebCrawlingURL>();
		Query query = entitymanager.createQuery("SELECT e FROM WebCrawlingURLJPA e");
		List<WebCrawlingURLJPA> wcuJPAs = (List<WebCrawlingURLJPA>) query.getResultList();
		for(WebCrawlingURLJPA wcuJPA: wcuJPAs) {
			wcus.add(wcuJPA.getWebCrawlingURL());
		}
		return wcus;
		
	}

}
