/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 23/12/2013
 *
 */
package uoc.tfg.manager.webfeed;

import java.io.IOException;
import java.util.Observable;
import java.util.Observer;
import java.util.Queue;

import org.apache.lucene.document.Document;

import com.sun.syndication.feed.synd.SyndEntry;

import uoc.tfg.exception.WebFeedException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.WebContentJPA;
import uoc.tfg.model.document.IIndexHandler;
import uoc.tfg.model.document.Index;
import uoc.tfg.model.document.IndexHandler;
import uoc.tfg.model.news.IWebContentPuller;
import uoc.tfg.model.news.WebContent;
import uoc.tfg.model.news.webfeed.IWebFeedReader;
import uoc.tfg.model.news.webfeed.WebFeedReader;
import uoc.tfg.model.news.webfeed.WebFeedSubscription;


public class WebFeedReaderManager extends Observable implements IWebFeedReaderManager, IWebContentPuller {

	private IWebFeedReader webFeedReader;	
	private IIndexHandler indexManager;	
	
	public WebFeedReaderManager(WebFeedSubscription wfs, Index index, Observer o) throws IOException {
		this.webFeedReader = new WebFeedReader(wfs, this);
		this.indexManager = new IndexHandler(index.getPath());
		this.addObserver(o);
	}


	@Override
	public void startWebFeedReader() throws WebFeedException {
		this.webFeedReader.startWebFeedReader();
	}
	
	@Override
	public void pausetWebFeedReader() {
		this.webFeedReader.pauseWebFeedReader();
	}

	@Override
	public void resumeWebFeedReader() {
		this.webFeedReader.resumeWebFeedReader();
	}
	
	@Override
	public void stopWebFeedReader() {
		this.webFeedReader.stopWebFeedReader();
	}
	
	@Override
	public String getName() {
		return this.webFeedReader.getName();
	}

	@Override
	public String getStatus() {
		return this.webFeedReader.getStatus().toString();
	}
	
	@Override
	public Queue<SyndEntry> getLastEntries() {
		return this.webFeedReader.getLastEntries();
	}

	@Override
	public void update(WebContent content) {
		WebContentJPA cJPA = new WebContentJPA(content);
		if(content != null) {
			try {
				DatabaseManager.persist(cJPA);
			} catch(Exception e) {}
		}
	}
	
	@Override
	public void update(Document doc) {
		
		try {
			indexManager.createIndexWriter();
			indexManager.addDocument(doc);
		} catch (IOException e) {}
	}
	
	@Override
	public void update(Observable arg0, Object arg1) {
		this.setChanged();
		this.notifyObservers();
	}


	@Override
	public void setIndex(Index i) throws IOException {

		this.indexManager.openDirectory(i.getPath());
	}
	
}