/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 23/12/2013
 *
 */
package uoc.tfg.manager.webfeed;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import uoc.tfg.exception.DatabaseException;
import uoc.tfg.jpa.DatabaseManager;
import uoc.tfg.jpa.news.webfeed.WebFeedURLJPA;
import uoc.tfg.model.news.webfeed.WebFeedURL;

/*
 *
 */
public class WebFeedURLManager implements IWebFeedURLManager {
	

	
	@Override
	public void newWebFeedURL(WebFeedURL wfu) {
		
    	DatabaseManager.persist(new WebFeedURLJPA(wfu));
    	
	}

	
	@Override
	public void refreshWebFeedURL(WebFeedURL wfu) {
		
    	DatabaseManager.refresh(new WebFeedURLJPA(wfu));
		
	}
	
	@Override
	public void editWebFeedURL(WebFeedURL wfu) {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebFeedURLJPA wfujpa = entitymanager.find(WebFeedURLJPA.class, wfu.getUrlText());
		wfujpa.setUrlName(wfu.getName());
		wfujpa.setUrlDescription(wfu.getDescription());
		wfujpa.setUrlFormat(wfu.getWebFeedformat().toString());
		wfujpa.setLastpoll(wfu.getLastPoll());
    	entitymanager.getTransaction().commit();
    	entitymanager.close();
	}
		
	@Override
	public void deleteWebFeedURL(String url) throws DatabaseException {
		
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		entitymanager.getTransaction().begin();
		WebFeedURLJPA wfuJPA = entitymanager.find(WebFeedURLJPA.class, url);
		if(wfuJPA == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			entitymanager.remove(wfuJPA);
		}
		entitymanager.getTransaction().commit();
		entitymanager.close();	
	}

	
	@Override
	public WebFeedURL getWebFeedURL(String url) throws DatabaseException, MalformedURLException {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		WebFeedURLJPA wfuJPA = entitymanager.find(WebFeedURLJPA.class, url);
		if(wfuJPA == null) {
			throw new DatabaseException(DatabaseException.ENTRY_NOT_FOUND);
		} else {
			return wfuJPA.getWebFeedURL();
		}
		
	}

	
	@Override
	public List<WebFeedURL> getAllWebFeedURL() throws MalformedURLException {
		EntityManager entitymanager = 
				DatabaseManager.getFactory().createEntityManager();
		List<WebFeedURL> wfus = new ArrayList<WebFeedURL>();
		Query query = entitymanager.createQuery("SELECT e FROM WebFeedURLJPA e");
		List<WebFeedURLJPA> wfuJPAs = (List<WebFeedURLJPA>) query.getResultList();
		for(WebFeedURLJPA wfuJPA: wfuJPAs) {
			wfus.add(wfuJPA.getWebFeedURL());
		}
		return wfus;
	}


}
