package uoc.tfg.model;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.IOUtils;




/**
 * Trading platform
 * 
 * @author Javier Sanzol
 * 
 * 2012-09-03 v.1
 *
 */

public class DateParser {
	
	private final static SimpleDateFormat[] sdfCollection = {
		new SimpleDateFormat("d/MM/yyyy HH:mm"),
		new SimpleDateFormat("d/MM/yyyy HH:mm:ss"),
		new SimpleDateFormat("d/MM/yyyy HH.mm"),
		new SimpleDateFormat("d/MM/yyyy HH.mm.ss"),
		new SimpleDateFormat("d-MM-yyyy HH:mm"),
		new SimpleDateFormat("d-MM-yyyyy HH:mm:ss"),
		new SimpleDateFormat("d-MM-yyyy HH.mm"),
		new SimpleDateFormat("d-MM-yyyy HH.mm.ss"),
		new SimpleDateFormat("dd/MM/yyyy HH:mm"),
		new SimpleDateFormat("dd/MM/yyyy HH:mm:ss"),
		new SimpleDateFormat("dd/MM/yyyy HH.mm"),
		new SimpleDateFormat("dd/MM/yyyy HH.mm.ss"),
		new SimpleDateFormat("dd-MM-yyyy HH:mm"),
		new SimpleDateFormat("dd-MM-yyyyy HH:mm:ss"),
		new SimpleDateFormat("dd-MM-yyyy HH.mm"),
		new SimpleDateFormat("dd-MM-yyyy HH.mm.ss"),
		new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"),
		new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ"),
		new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssXXX")};
	
	private final static String[] dateRegex = {
		"(\\d{1,2})/(\\d{1,2})/(\\d{4})",
		"(\\d{1,2})-(\\d{1,2})-(\\d{4})",
		"(\\d{1,2}).(\\d{1,2}).(\\d{4})"	
	};
	
	private final static String[] timeRegex = {
		"(\\d{1,2}):(\\d{2})",
		"(\\d{1,2}).(\\d{2})"
	};
		
	private final static SimpleDateFormat timestamp1 = 
			new SimpleDateFormat("d/MM/yyyy HH:mm");
	
	
	
	private final static SimpleDateFormat timestamp2 = 
			new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	
	private final static SimpleDateFormat timestamp3 = 
			new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
	
	private final static SimpleDateFormat timestamp4 = 
			new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssXXX");	
	
	private final static SimpleDateFormat date1 = 
			new SimpleDateFormat("yyyy-MM-dd");
	
	private final static SimpleDateFormat date2 = 
			new SimpleDateFormat("yyyy/MM/dd");
	
	private final static SimpleDateFormat date3 = 
			new SimpleDateFormat("yyyy.MM.dd");
	
	private final static SimpleDateFormat date4 = 
			new SimpleDateFormat("dd-MM-yyyy");
	
	private final static SimpleDateFormat ddMMyyyy = 
			new SimpleDateFormat("dd/MM/yyyy");
	
	private final static SimpleDateFormat date6 = 
			new SimpleDateFormat("dd.MM.yyyy");
	
	private final static SimpleDateFormat timeSdf = 
			new SimpleDateFormat("HH:mm:ss");
	
	private final static SimpleDateFormat timeWithTimezoneSdf = 
			new SimpleDateFormat("HH:mm:ssZZZ");
	
	
	
	
	
	private final static SimpleDateFormat timestampSdfLS = 
			new SimpleDateFormat("dd/MM/yyHH:mm:ss");
	
	private final static SimpleDateFormat timestampWithTimezone1 = 
			new SimpleDateFormat("yyyy-MM-dd HH:mm:ssZZZ");
	
	private final static SimpleDateFormat timestampWithTimezone2 = 
			new SimpleDateFormat("EEE, dd MMM YYYY HH:mm:ss zzz");
	
	
	public static Date parse(String date) {
	
		Date d = null;
		
		if(date == null) {
			return null;
		}
		for(SimpleDateFormat sdf: sdfCollection) {
			try {
				d = sdf.parse(date);
			} catch (ParseException e) {}
			if(d != null) return d;
		}
		
		return null;
		
	}
	
	public static String parseInputStream(InputStream inputStream) throws IOException, ParseException {
		
		String date = null;
		
		StringWriter writer = new StringWriter();
		IOUtils.copy(inputStream, writer, "utf-8");
		String inputString = writer.toString().replaceAll("[(\\r\\n)\\n\\t]", "");
		
		for(String dr: dateRegex) {
			Pattern patternDate = Pattern.compile(dr);
			for(String tr: timeRegex) {
				Pattern patternTime = Pattern.compile(tr);
				Pattern pattern1 = Pattern.compile(patternTime.pattern()+"(.{0,20})"+patternDate.pattern());
				if(inputString.matches("(.)*"+pattern1.pattern()+"(.)*")){
					
					Matcher result = pattern1.matcher(inputString);
					result.find();
					Matcher mDate = patternDate.matcher(result.group(0));
		    		Matcher mTime = patternTime.matcher(result.group(0));
		    		mDate.find();
		    		mTime.find();
		    		date = mDate.group(0)+" "+mTime.group(0);
		    		break;
		    	}
				Pattern pattern2 = Pattern.compile(patternDate.pattern()+"(.{0,20})"+patternTime.pattern());
				if(inputString.matches("(.)*"+pattern2.pattern()+"(.)*")){
					
					Matcher result = pattern2.matcher(inputString);
					result.find();
					Matcher mDate = patternDate.matcher(result.group(0));
		    		Matcher mTime = patternTime.matcher(result.group(0));
		    		mDate.find();
		    		mTime.find();
		    		date = mDate.group(0)+" "+mTime.group(0);
		    		break;
		    	}
			}
		}
		
		//Pattern patternDate = Pattern.compile("(\\d{1,2})/(\\d{1,2})/(\\d{4})");
		//Pattern patternTime = Pattern.compile("(\\d{1,2}):(\\d{1,2})");
		//Pattern pattern = Pattern.compile(patternTime.pattern()+"(.{0,20})"+patternDate.pattern());
				
		

		return date;
	}
	
	public static SimpleDateFormat getTimeFormat() {
		return timeSdf;
	}
	
	public static SimpleDateFormat getTimeWithTimezoneFormat() {
		return timeWithTimezoneSdf;
	}
	
	public static SimpleDateFormat getTimestampWithTimezoneFormat() {
		return timestampWithTimezone1;
	}
	
	public static SimpleDateFormat getTimestampFormatLS() {
		return timestampSdfLS;
	}

	public static SimpleDateFormat getTimestampFormat() {
		// TODO Auto-generated method stub
		return null;
	}

	public static SimpleDateFormat getDateFormatddMMyyyy() {
		// TODO Auto-generated method stub
		return ddMMyyyy;
	}

}
