package uoc.tfg.model;

public enum Interval {

	ONE_MINUTE(1),
	TWO_MINUTE(2),
	THREE_MINUTE(3),
	FIVE_MINUTE(4),
	TEN_MINUTE(5),
	FIVETEEN_MINUTE(6),
	THIRTY_MINUTE(7),
	ONE_HOUR(8),
	ONE_DAY(9),
	ONE_WEEK(10),
	ONE_MONTH(11),
	ONE_YEAR(12);
	
	
private final int value;
	
	/**
	 * Constructor
	 * @param value
	 */
	Interval(int value) {
		this.value = value;
	}
	
	/**
	 * 
	 * @return
	 */
	public int getValue() {
		return this.value;
	}
	
	/**
	 * 
	 * @param value
	 * @return
	 */
	public static Interval valueOf(int value) {
		switch(value) {
		case 1:
			return Interval.ONE_MINUTE;
		case 2:
			return Interval.TWO_MINUTE;
		case 3:
			return Interval.THREE_MINUTE;
		case 4:
			return Interval.FIVE_MINUTE;
		case 5:
			return Interval.TEN_MINUTE;
		case 6:
			return Interval.FIVETEEN_MINUTE;
		case 7:
			return Interval.THIRTY_MINUTE;
		case 8:
			return Interval.ONE_HOUR;
		case 9:
			return Interval.ONE_DAY;
		case 10:
			return Interval.ONE_WEEK;
		case 11:
			return Interval.ONE_MONTH;
		case 12:
			return Interval.ONE_YEAR;
		default:
			return null;
		}
	}
	
	
	public long getMilliseconds() {
		
		switch(this) {
			case ONE_MINUTE:
				return 60000;
			case TWO_MINUTE:
				return 120000;
			case THREE_MINUTE:
				return 180000;
			case FIVE_MINUTE:
				return 300000;
			case TEN_MINUTE:
				return 600000;
			case FIVETEEN_MINUTE:
				return 900000;
			case THIRTY_MINUTE:
				return 1800000;
			case ONE_HOUR:
				return 3600000;
			case ONE_DAY:
				return 86400000;
			case ONE_WEEK:
				return 0;
			case ONE_MONTH:
				return 0;
			case ONE_YEAR:
				return 0;
			default:
				return 0;
		}
	}
	
	public static Interval getValueOf(String value) {
		if(value.equals("1 minute")) return Interval.ONE_MINUTE;
		else if(value.equals("2 minute")) return Interval.TWO_MINUTE;
		else if(value.equals("3 minute")) return Interval.THREE_MINUTE;
		else if(value.equals("5 minutes")) return Interval.FIVE_MINUTE;
		else if(value.equals("10 minutes")) return Interval.TEN_MINUTE;
		else if(value.equals("15 minutes")) return Interval.FIVETEEN_MINUTE;
		else if(value.equals("30 minutes")) return Interval.THIRTY_MINUTE;
		else if(value.equals("1 hour")) return Interval.ONE_HOUR;
		else if(value.equals("Daily")) return Interval.ONE_DAY;
		else if(value.equals("Weekly")) return Interval.ONE_WEEK;
		else if(value.equals("Monthly")) return Interval.ONE_MONTH;
		else if(value.equals("Annual")) return Interval.ONE_YEAR;
		else return null;
	}
	
	public static Interval getValueOf(long value) {
		if(value == 60000) return Interval.ONE_MINUTE;
		else if(value == 120000) return Interval.TWO_MINUTE;
		else if(value == 180000) return Interval.THREE_MINUTE;
		else if(value == 300000) return Interval.FIVE_MINUTE;
		else if(value == 600000) return Interval.TEN_MINUTE;
		else if(value == 900000) return Interval.FIVETEEN_MINUTE;
		else if(value == 1800000) return Interval.THIRTY_MINUTE;
		else if(value == 3600000) return Interval.ONE_HOUR;
		else if(value == 86400000) return Interval.ONE_DAY;
		else return null;
	}
	
	
	/**
	 * Text representation of a resolution data bar
	 * @return
	 */
	@Override
	public String toString() {
		
		switch(this) {
			case ONE_MINUTE:
				return "1 minute";
			case TWO_MINUTE:
				return "2 minute";
			case THREE_MINUTE:
				return "3 minute";
			case FIVE_MINUTE:
				return "5 minutes";
			case TEN_MINUTE:
				return "10 minutes";
			case FIVETEEN_MINUTE:
				return "15 minute";
			case THIRTY_MINUTE:
				return "30 minutes";
			case ONE_HOUR:
				return "1 hour";
			case ONE_DAY:
				return "Daily";
			case ONE_WEEK:
				return "Weekly";
			case ONE_MONTH:
				return "Monthly";
			case ONE_YEAR:
				return "Annual";
			default:
				return "";
		}
	}
	
}