package uoc.tfg.model.document;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryparser.classic.MultiFieldQueryParser;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

import uoc.tfg.model.textanalysis.SpanishTokenizer;

public class IndexHandler implements IIndexHandler{
	
	private Analyzer analyzer;
	private Directory directory;
	private DirectoryReader directoryReader;
	private IndexWriter indexWriter;
	
	public IndexHandler(File path) throws IOException {
		this.directory = FSDirectory.open(path);
		this.analyzer = new SpanishTokenizer();
	}
	
	@Override
	public void openDirectory(File path) throws IOException {
		this.directory = FSDirectory.open(path);
	}

	@Override
	public void createIndexWriter() throws IOException {
		
		IndexWriterConfig config = new IndexWriterConfig(Version.LUCENE_46, analyzer);
		indexWriter = new IndexWriter(directory, config);
	}
	
	@Override
	public void addDocument(Document doc) throws IOException {
		
	    indexWriter.addDocument(doc);
	    indexWriter.close();
	}
	
	@Override
	public void deleteDocument(Document doc) {
		
	}
	
	@Override
	public void updateDocument(Document doc) {
		
	}

	@Override
	public List<Document> searchIndex(String[] fields, String[] queries, String[] logicals) throws ParseException, IOException {
		
		this.directoryReader = DirectoryReader.open(this.directory);
	    IndexSearcher indexSearcher = new IndexSearcher(this.directoryReader);
	    BooleanClause.Occur[] bq = new BooleanClause.Occur[logicals.length];
	    for(int i=0; i<queries.length; i++) {
	    	if(logicals[i].equals("AND")) bq[i] = BooleanClause.Occur.MUST;
	    	else if(logicals[i].equals("OR")) bq[i] = BooleanClause.Occur.SHOULD;
	    	else if(logicals[i].equals("AND NOT")) bq[i] = BooleanClause.Occur.MUST_NOT;
	    }
	    
	    Query query = MultiFieldQueryParser.parse(Version.LUCENE_46, queries, fields, bq, this.analyzer);
	    ScoreDoc[] hits = indexSearcher.search(query, null, 1000).scoreDocs;
	    List<Document> docs = new ArrayList<Document>();
	    
	    for (int i = 0; i < hits.length; i++) {
	    	docs.add(indexSearcher.doc(hits[i].doc));
	    }
	    
	    this.directoryReader.close();
	    
	    return docs;
	}

	@Override
	public List<Document> getAllDocuments() throws IOException {
		
		this.directoryReader = DirectoryReader.open(this.directory);
	    List<Document> docs = new ArrayList<Document>();
	    
	    for (int i=0; i<this.directoryReader.maxDoc(); i++) {

	        docs.add(this.directoryReader.document(i));
	    }
	    
	    this.directoryReader.close();
	    
	    return docs;
	}

	@Override
	public int getNumDocuments() throws IOException {

		this.directoryReader = DirectoryReader.open(this.directory);
		int num = this.directoryReader.numDocs();
		this.directoryReader.close();
	   
		return num;
	}
	
}
