package uoc.tfg.model.marketdata;

import java.io.Serializable;
import java.util.Date;


/**
 * 
 * @author Javier
 *
 */

/*
 * Represents a financial instrument: stock, future, forex, etc...
 */
public class Instrument implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Instrument symbol
	 */
	private String symbol;
	/**
	 * Instrument name
	 */
	private String name;
	/**
	 * Instrument ISIN; international code
	 */
	private String isin;
	/**
	 * Instrument market symbol
	 */
	private String marketSymbol;
	/**
	 * Instrument market country
	 */
	private String marketCountry;
	/**
	 * The first date for which the data is available
	 */
	private Date dataFrom;
	/**
	 * The last date for which the data is available
	 */
	private Date dataTo;
	/**
	 * 
	 */
	private boolean canceled;
	
	/**
	 * Constructors
	 */
	public Instrument() {
		this(null, null, null, null, null, null, null, false);
	}
	
	public Instrument(String symbol, String name, String isin, String marketSymbol, 
			String marketCountry) {
		this(symbol, name, isin, marketSymbol, marketCountry, null, null, false);
	}
	
	public Instrument(String symbol, String name, String isin, String marketSymbol, 
			String marketCountry, Date startDate, Date endDate, boolean canceled) {
		this.symbol = symbol;
		this.name = name;
		this.isin = isin;
		this.marketSymbol = marketSymbol;
		this.marketCountry = marketCountry;
		this.dataFrom = startDate;
		this.dataTo = endDate;
		this.canceled = canceled;
	}
	
	/**
	 * Returns true if this is equals to obj
	 */
	@Override
	public boolean equals(Object obj) {
		
		Instrument instrument = (Instrument) obj;
		if(obj == null)
		      return false;
		if(!(obj instanceof Instrument))
			return false;

		return (this.symbol.equals(instrument.symbol) &&
				this.marketSymbol.equals(instrument.marketSymbol) &&
				this.marketCountry.equals(instrument.marketCountry));
		
	}
	

	/**
	 * SETTERS
	 * 
	 * Set name
	 */
	public void setName(String name){
		this.name = name;
	}
	/**
	 * Set symbol
	 */
	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}
	/**
	 * Set isin
	 */
	public void setIsin(String isin) {
		this.isin = isin;
	}
	/**
	 * Set market symbol
	 */
	public void setMarketSymbol(String market) {
		this.marketSymbol = market;
	}
	/**
	 * Set market country
	 */
	public void setMarketCountry(String country) {
		this.marketCountry = country;
	}
	/**
	 * Set start date
	 */
	public void setDataAvailableFrom(Date dataFrom) {
		this.dataFrom = dataFrom;
	}
	/**
	 * Set end date
	 */
	public void setDataAvailableTo(Date dataTo) {
		this.dataTo = dataTo;
	}
	/**
	 * 
	 */
	public void setIsCanceled(boolean canceled) {
		this.canceled = canceled;
	}
	
	/**
	 * Getters
	 * 
	 */
	/**
	 * Get name
	 * @return
	 */
	public String getName() {
		return this.name;
	}
	/**
	 * Get symbol
	 */
	public String getSymbol() {
		return this.symbol;
	}	
	/**
	 * Get isin
	 */
	public String getIsin() {
		return this.isin;
	}
	/**
	 * Get asset market
	 */
	public String getMarketSymbol() {
		return this.marketSymbol;
	}
	/**
	 * Get asset market country
	 */
	public String getMarketCountry() {
		return this.marketCountry;
	}
	/**
	 * Get start date
	 */
	public Date getDataAvailableFrom() {
		return this.dataFrom;
	}
	/**
	 * Get end date
	 */
	public Date getDataAvailableTo() {
		return this.dataTo;
	}
	/**
	 * 
	 */
	public boolean isCanceled() {
		return this.canceled;
	}
	
	/**
	 * Text representation of an equity object
	 */
	public String toString() {
		
		StringBuffer sb = new StringBuffer();
	
		sb.append(this.symbol);
		sb.append(" ");
		sb.append(this.name);
		sb.append(" ");
		sb.append(this.marketSymbol);
		sb.append(" ");
		sb.append(this.marketCountry);
		
		return sb.toString();
		
	}
	
	
}
























