package uoc.tfg.model.marketdata;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import uoc.tfg.model.marketdata.MarketCalendar.MarketSchedule;


/*
 * Represents a market
 */
public class Market implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private String symbol;
	
	private String name;
	
	private InstrumentType type;
	
	private String city;
	
	private String country;
	
	private MarketCalendar calendar;
	
	
	/**
	 * Constructors
	 */
	public Market() {
		this(null, null, null, null, null, null);		
	}
	
	public Market(String symbol, String name, InstrumentType type, String city, 
			String country, MarketCalendar calendar) {
		this.symbol = symbol;
		this.name = name;
		this.type = type;
		this.city = city;
		this.country = country;
		this.calendar = calendar;		
	}
	
	/**
	 * Returns true if this is equals to obj
	 */
	@Override
	public boolean equals(Object obj) {
		
		Market market = (Market) obj;
		if(obj == null)
		      return false;
		if(!(obj instanceof Market))
			return false;

		return (this.symbol.equals(market.symbol) &&
				this.country.equals(market.country));
		
	}
	

	/**
	 * SETTERS
	 */
	/**
	 * Sets symbol
	 * @param symbol
	 */
	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}
	/**
	 * Set name
	 * @param name
	 */
	public void setName(String name){
		this.name = name;
	}
	/**
	 * Set type of market
	 */
	public void setMarketType(InstrumentType type) {
		this.type = type;
	}
	/**
	 * Set city
	 */
	public void setCity(String city) {
		this.city = city;
	}
	/**
	 * Set country
	 */
	public void setCountry(String country) {
		this.country = country;
	}
	/**
	 * Set close time
	 */
	public void setCalendar(MarketCalendar calendar) {
		this.calendar = calendar;
	}
	
	/**
	 * GETTERS
	 */
	
	/**
	 * Get symbol
	 */
	public String getSymbol() {
		return this.symbol;
	}	
	/**
	 * Get name
	 * @return
	 */
	public String getName() {
		return this.name;
	}
	/**
	 * Get type
	 */
	public InstrumentType getInstrumentType() {
		return this.type;
	}
	/**
	 * Get city
	 */
	public String getCity() {
		return this.city;
	}
	/**
	 * Get country
	 */
	public String getCountry() {
		return this.country;
	}
	/**
	 * Get the market calendar
	 */
	public MarketCalendar getCalendar() {
		return this.calendar;
	}
	/**
	 * Get markets weekly schedule
	 */
	public MarketSchedule getSchedule() {
		return this.calendar.getSchedule();
	}
	
	public TimeZone getTimeZone() {
		return this.calendar.getTimeZone();
	}
	
	public Calendar currentTime() {
		return this.calendar.currentMarketTime();
	}
	
	public boolean isTradingDay(Date date) {
		return (!this.calendar.isDayOff(date) && 
				this.calendar.getSchedule().isSessionDay(date));
	}
	
	public boolean isTradingDay(Calendar date) {
		return this.isTradingDay(date.getTime());
	}
	
	public MarketSession getNextMarketSession() {
		MarketSession session = new MarketSession(this);
		session.nextTradingSession();
		return session;
	}

	
	/**
	 * Text representation of an equity object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		
		sb.append(this.symbol);
		sb.append("\t");
		sb.append(this.name);
		sb.append("\t");
		sb.append(this.type.toString());
		sb.append("\t");
		sb.append(this.city);
		sb.append("\t");
		sb.append(this.country);
		
		return sb.toString();
	}


}
