package uoc.tfg.model.marketdata;


import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.TimeZone;
import java.util.Vector;

import uoc.tfg.model.DateParser;



/*
 * Representation of the market's calendar. This class mannage the market time zone and
 * the days off.
 */
public class MarketCalendar extends GregorianCalendar implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private final MarketSchedule schedule;
	
	private final Vector<Date[]> daysOff;
	
	
	
	public MarketCalendar(TimeZone timezone, boolean sunday, boolean monday, boolean tuesday, 
			boolean wednesday, boolean thursday, boolean friday, boolean saturday,
			long openOrdering, long closeOrdering, long openTrading, 
			long closeTrading, long openPricing, long closePricing) {
		this.setTimeZone(timezone);
		this.daysOff = new Vector<Date[]>();
		this.schedule = new MarketSchedule(this, sunday, monday, tuesday, wednesday, thursday, friday, 
				saturday, openOrdering, closeOrdering, openTrading, closeTrading, openPricing, closePricing);
	}
	
	public void addDayOff(Date d1, Date d2) {
		Date[] date = {d1, d2};
		this.daysOff.add(date);
	}
	
	public void addDayOff(Calendar d1, Calendar d2) {
		Date[] date = {d1.getTime(), d2.getTime()};
		this.daysOff.add(date);
	}
	
	/**
	 * GETTERS
	 */
	
	/**
	 * 
	 * @return
	 */
	public MarketSchedule getSchedule() {
		return this.schedule;
	}
	
	public Vector<Date[]> getDaysOff() {
		return this.daysOff;
	}

	
	public boolean isDayOff(Date time) {
		
		if(!this.daysOff.isEmpty()) {
			Date[] data;
			int positionBotton = 0;
			int positionTop = this.daysOff.size()-1;
			int position = (this.daysOff.size()-positionBotton)/2;
			data = this.daysOff.get(position);
		
			while((position != positionBotton) && (position != positionTop)) {
				
				if((data[0].getTime()) == (time.getTime())) {
					return true;
				} else if((data[0].getTime()) > (time.getTime())) {
					positionTop = position;
					position = position - (position-positionBotton)/2;
					data = this.daysOff.get(position);
				} else if((data[0].getTime()) < (time.getTime())) {
					positionBotton = position;
					position = position + (positionTop-position)/2;
					data = this.daysOff.get(position);
				}
				
			}
			
			if((time.getTime() < this.daysOff.get(positionBotton)[0].getTime()) || 
					(time.getTime() > this.daysOff.get(positionTop)[1].getTime()))
				return false;
			
			else if((time.getTime() <= this.daysOff.get(positionBotton)[1].getTime()) || 
					(time.getTime() >= this.daysOff.get(positionTop)[0].getTime()))
				return true;
			
			else
				return false;
		
		} 
		
		return false;		

	}
	
	
	public boolean isDayOff(Calendar time) {
		return this.isDayOff(time.getTime());
	}
	
	public boolean isClosed(Date time) {
		return (this.isDayOff(time) || 
				!this.schedule.isSessionDay(time) || 
				!this.schedule.isTradingTime(time));
	}
	
	public boolean isClosed(Calendar time) {
		return this.isClosed(time.getTime());
	}
	
	
	public SimpleDateFormat getTimeFormat() {
		SimpleDateFormat timeSdf = DateParser.getTimeFormat();
		timeSdf.setTimeZone(this.getTimeZone());
		return timeSdf;
	}
	
	
	public SimpleDateFormat getDateFormat() {
		SimpleDateFormat dateSdf = DateParser.getDateFormatddMMyyyy();
		dateSdf.setTimeZone(this.getTimeZone());
		return dateSdf;
	}
	
	public SimpleDateFormat getTimestampFormat() {
		SimpleDateFormat timestampSdf = DateParser.getTimestampFormat();
		timestampSdf.setTimeZone(this.getTimeZone());
		return timestampSdf;
	}
	
	public SimpleDateFormat getTimestampFormatLS() {
		SimpleDateFormat timestampSdfLS = DateParser.getTimestampFormatLS();
		timestampSdfLS.setTimeZone(this.getTimeZone());
		return timestampSdfLS;
	}
	
	public Calendar currentMarketTime() {
		Calendar calendar = Calendar.getInstance();
		return this.changeToMarketTimeZone(calendar);
	}
	
	public Calendar changeToMarketTimeZone(Calendar instant) {
		
		Calendar calendar = new GregorianCalendar(this.getTimeZone());
		calendar.setTimeInMillis(instant.getTimeInMillis());
		
		return calendar;
		
	}
	
	public Calendar changeToMarketTimeZone(Date instant) {
		
		Calendar calendar = new GregorianCalendar(this.getTimeZone());
		calendar.setTimeInMillis(instant.getTime());
		
		return calendar;
		
	}
	
	public class MarketSchedule implements Serializable {
		
		private static final long serialVersionUID = 1L;
		
		private final MarketCalendar marketCalendar;
		
		private final Hashtable<WeekDay, Boolean> sessions;
		
		private final Time openOrdering;
		
		private final Time closeOrdering;
		
		private final Time openTrading;
		
		private final Time closeTrading;
		
		private final Time openPricing;
		
		private final Time closePricing;
		
		
		/**
		 * Constructor
		 * @throws ApplicationException 
		 */		
		public MarketSchedule(MarketCalendar marketCalendar, boolean sunday, boolean monday, boolean tuesday, 
				boolean wednesday, boolean thursday, boolean friday, boolean saturday,
				long openOrdering, long closeOrdering, long openTrading, 
				long closeTrading, long openPricing, long closePricing) {
			this(marketCalendar, sunday, monday, tuesday, wednesday, thursday, friday, saturday,
					new Time(openOrdering), new Time(closeOrdering), new Time(openTrading), 
					new Time(closeTrading), new Time(openPricing), new Time(closePricing));
		}
		
		public MarketSchedule(MarketCalendar marketCalendar, boolean sunday, boolean monday, boolean tuesday, 
				boolean wednesday, boolean thursday, boolean friday, boolean saturday,
				Time openOrdering, Time closeOrdering, Time openTrading, 
				Time closeTrading, Time openPricing, Time closePricing) {
			this.marketCalendar = marketCalendar;
			this.sessions = new Hashtable<WeekDay, Boolean>();
			this.sessions.put(WeekDay.SUNDAY, sunday);
			this.sessions.put(WeekDay.MONDAY, monday);
			this.sessions.put(WeekDay.TUESDAY, tuesday);
			this.sessions.put(WeekDay.WEDNESDAY, wednesday);
			this.sessions.put(WeekDay.THURSDAY, thursday);
			this.sessions.put(WeekDay.FRIDAY, friday);
			this.sessions.put(WeekDay.SATURDAY, saturday);
			this.openOrdering = openOrdering;
			this.closeOrdering = closeOrdering;
			this.openTrading = openTrading;
			this.closeTrading = closeTrading;
			this.openPricing = openPricing;
			this.closePricing = closePricing;		
		}
		
		public MarketCalendar getMarketCalendar() {
			return this.marketCalendar;
		}
		
		public boolean isSessionDay(Date day) {
			
			Calendar testDay = new GregorianCalendar(this.marketCalendar.getTimeZone());
			testDay.setTime(day);
			return this.isSessionDay(testDay);

		}
		
		public boolean isSessionDay(Calendar testDay) {
			
			switch(testDay.get(Calendar.DAY_OF_WEEK)) {
			case Calendar.SUNDAY:
				return (boolean)this.sessions.get(WeekDay.SUNDAY);
			case Calendar.MONDAY:
				return (boolean)this.sessions.get(WeekDay.MONDAY);
			case Calendar.TUESDAY:
				return (boolean)this.sessions.get(WeekDay.TUESDAY);
			case Calendar.WEDNESDAY:
				return (boolean)this.sessions.get(WeekDay.WEDNESDAY);
			case Calendar.THURSDAY:
				return (boolean)this.sessions.get(WeekDay.THURSDAY);
			case Calendar.FRIDAY:
				return (boolean)this.sessions.get(WeekDay.FRIDAY);
			case Calendar.SATURDAY:
				return (boolean)this.sessions.get(WeekDay.SATURDAY);
			}
			
			return false;
			
		}
		
		public boolean isPreOrderingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
			
			Time time = new Time(calendar);
			
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() < this.getOpenOrdering().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
			
		}
		
		public boolean isPreOrderingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isPreOrderingTime(testDay);
					
		}
		
		public boolean isOrderingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
			
			Time time = new Time(calendar);
			
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() >= this.getOpenOrdering().getSecondsFromMidNight() && 
						time.getSecondsFromMidNight() < this.getCloseOrdering().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
			
		}
		
		public boolean isOrderingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isOrderingTime(testDay);
					
		}
		
		public boolean isPostOrderingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
			
			Time time = new Time(calendar);
			
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() >= this.getCloseOrdering().getSecondsFromMidNight()) {
					System.out.println(time.getSecondsFromMidNight());
					System.out.println(this.getCloseOrdering().getSecondsFromMidNight());
					return true;			
				}
			}
			
			return false;
			
		}
		
		public boolean isPostOrderingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isPostOrderingTime(testDay);
					
		}
		
		public boolean isPreTradingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
						
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() < this.getOpenTrading().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isPreTradingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isPreTradingTime(testDay);
					
		}
		
		public boolean isTradingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
						
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() >= this.getOpenTrading().getSecondsFromMidNight() && 
						time.getSecondsFromMidNight() < this.getCloseTrading().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isTradingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isTradingTime(testDay);
					
		}
		
		public boolean isPostTradingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
						
			if(this.isSessionDay(calendar)) {
				if(time.getSecondsFromMidNight() >= this.getCloseTrading().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isPostTradingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			return this.isPostTradingTime(testDay);
					
		}
		
		public boolean isPrePricingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
			
			if(this.isSessionDay(instant)) {
				if(time.getSecondsFromMidNight() < this.getOpenPricing().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isPrePricingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			
			return this.isPrePricingTime(testDay);
					
		}
		
		public boolean isPricingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
			
			if(this.isSessionDay(instant)) {
				if(time.getSecondsFromMidNight() >= this.getOpenPricing().getSecondsFromMidNight() && 
						time.getSecondsFromMidNight() < this.getClosePricing().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isPricingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			
			return this.isPricingTime(testDay);
					
		}
		
		public boolean isPostPricingTime(Calendar instant) {
			
			Calendar calendar = this.marketCalendar.changeToMarketTimeZone(instant);
						
			Time time = new Time(calendar);
			
			if(this.isSessionDay(instant)) {
				if(time.getSecondsFromMidNight() >= this.getClosePricing().getSecondsFromMidNight())
					return true;			
			}
			
			return false;
		}
		
		public boolean isPostPricingTime(Date instant) {
			
			Calendar testDay = new GregorianCalendar();
			testDay.setTime(instant);
			
			return this.isPostPricingTime(testDay);
					
		}
		
		
		/**
		 * GETTERS
		 */
		
		public Time getOpenOrdering() {
			return this.openOrdering;
		}
		
		public Time getCloseOrdering() {
			return this.closeOrdering;
		}
		
		public Time getOpenTrading() {
			return this.openTrading;
		}
		
		public Time getCloseTrading() {
			return this.closeTrading;
		}

		public Time getOpenPricing() {
			return this.openPricing;
		}
		
		public Time getClosePricing() {
			return this.closePricing;
		}

	}
	
}
