package uoc.tfg.model.marketdata;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import uoc.tfg.model.Interval;


public class MarketSession implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private final Market market;
	
	private final Calendar timelessCalendar;
	
	private final Calendar openOrdering;
	
	private final Calendar closeOrdering;
	
	private final Calendar openTrading;
	
	private final Calendar closeTrading;
	
	private final Calendar openPricing;
	
	private final Calendar closePricing;
	
	

	public MarketSession(Market market) {
		this(market, new Date());
	}
	
	public MarketSession(Market market, int year, int month, int day) {
		this(market, new GregorianCalendar(year, month-1, day, 0, 0, 0).getTime());
	}
	
	public MarketSession(Market market, Calendar calendar) {
		this(market, calendar.getTime());
	}
	
	public MarketSession(Market market, Date date) {
		
		if(market == null || date == null)
			throw new IllegalArgumentException();
		this.market = market;
		this.timelessCalendar = new GregorianCalendar(this.market.getTimeZone());
		this.openOrdering = new GregorianCalendar(this.market.getTimeZone());
		this.closeOrdering = new GregorianCalendar(this.market.getTimeZone());
		this.openTrading = new GregorianCalendar(this.market.getTimeZone());
		this.closeTrading = new GregorianCalendar(this.market.getTimeZone());
		this.openPricing = new GregorianCalendar(this.market.getTimeZone());
		this.closePricing = new GregorianCalendar(this.market.getTimeZone());
		Calendar calendar = new GregorianCalendar(this.market.getTimeZone());
		calendar.setTime(date);
		this.nextTradingSession(calendar);
		
	}
	
	public synchronized void nextTradingSession() {
		
		this.timelessCalendar.setTimeInMillis(this.timelessCalendar.getTimeInMillis() + 
				Interval.ONE_DAY.getMilliseconds());
		this.nextTradingSession(this.timelessCalendar);
		
	}

	private synchronized void nextTradingSession(Calendar calendar) {
		
		if(this.market.getSchedule().isPostOrderingTime(calendar) &&
				this.market.getSchedule().isPostPricingTime(calendar) &&
				this.market.getSchedule().isPostTradingTime(calendar)) {
			
			long millis = calendar.getTimeInMillis() + Interval.ONE_DAY.getMilliseconds();
			calendar.setTimeInMillis(millis);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			this.timelessCalendar.setTimeInMillis(calendar.getTimeInMillis());
		} else {
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			this.timelessCalendar.setTimeInMillis(calendar.getTimeInMillis());
		}
		
		while(!this.market.isTradingDay(this.timelessCalendar)) {
			this.timelessCalendar.setTimeInMillis(this.timelessCalendar.getTimeInMillis() + 
					Interval.ONE_DAY.getMilliseconds());
		}
		
		this.openOrdering.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getOpenOrdering().getMillisFromMidNight());
		
		this.closeOrdering.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getCloseOrdering().getMillisFromMidNight());
		
		this.openTrading.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getOpenTrading().getMillisFromMidNight());
		
		this.closeTrading.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getCloseTrading().getMillisFromMidNight());
		
		this.openPricing.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getOpenPricing().getMillisFromMidNight());
		
		this.closePricing.setTimeInMillis(
				this.timelessCalendar.getTimeInMillis() + 
				this.market.getSchedule().getClosePricing().getMillisFromMidNight());
		
	}
	
	public Calendar getTimelessCalendar() {
		return this.timelessCalendar;
	}
	
	public Calendar getOpenOrdering() {
		return this.openOrdering;
	}
	
		
	public Calendar getCloseOrdering() {
		return this.closeOrdering;
	}
	
		
	public Calendar getOpenTrading() {
		return this.openTrading;
	}
	
	
	public Calendar getCloseTrading() {
		return this.closeTrading;
	}

		
	public Calendar getOpenPricing() {
		return this.openPricing;
	}
	
	
	public Calendar getClosePricing() {
		return this.closePricing;
	}
	
	public String sessionDayToString() {
		return this.market.getCalendar().getDateFormat().format(this.getTimelessCalendar().getTime());
	}
	
	public boolean isSameSession(MarketSession session) {
		return this.timelessCalendar.getTimeInMillis() == 
				session.timelessCalendar.getTimeInMillis();
	}
		

}
