package uoc.tfg.model.marketdata;

import java.io.Serializable;
import java.util.Calendar;


/*
 * Inner class. Represents a daily time point
 */
public class Time implements Serializable {

	private static final long serialVersionUID = 1L;
	public static final String AM = "AM";
	public static final String PM = "PM";
	public static final String H24 = "24H";
	private final int hour;
    private final int minute;
    private final int second;
    private final String mode;

    
    public Time(int hour, int minute, int second) {
    	this(hour, minute, second, Time.H24);
    }
    
    public Time(Calendar calendar) {
    	this(calendar.get(Calendar.HOUR_OF_DAY), calendar.get(Calendar.MINUTE), calendar.get(Calendar.SECOND), Time.H24);
    }
    
    public Time(long secondsFromMidnight) { 
    	if(secondsFromMidnight < 0 || secondsFromMidnight > 86399)
    		throw new IllegalArgumentException("");
    	this.hour = (int) (secondsFromMidnight / 3600);
    	this.minute = (int) ((secondsFromMidnight % 3600) / 60);
    	this.second = (int) ((secondsFromMidnight % 3600) % 60);
    	this.mode = Time.H24;
    }
    
    public Time(int hour, int minute, int second, String mode) {
    	
    	this.hour = hour;
    	this.minute = minute;
    	this.second = second;
    	this.mode = mode;
    	
    	if(this.mode == Time.AM || this.mode == Time.PM || this.mode == Time.H24) {
    		if(this.mode == Time.AM || this.mode == Time.PM) {
        		if(this.hour < 0 || this.hour > 11) {
        			throw new IllegalArgumentException("");
        		} 
        	} else if(this.mode == Time.H24) {
        		if(this.hour < 0 || this.hour > 23)
        			throw new IllegalArgumentException("");
        	}
    	} else {
    		throw new IllegalArgumentException("");
    	}
    	
    	if(this.minute < 0 || this.minute > 59)
    		throw new IllegalArgumentException("");
    	
    	if(this.second < 0 || this.second > 59)
    		throw new IllegalArgumentException("");
    }
    
       
	public int getHour() {
        return hour;
    }

    public int getMinute() {
        return minute;
    }

    public int getSecond() {
        return second;
    }
    
    public long getSecondsFromMidNight() {
    	if(this.mode.equals(Time.H24) || this.mode.equals(Time.AM)) {
    		return ((hour * 3600) + (minute * 60) + second);
    	} else if(this.mode.equals(Time.PM)) {
    		return ((43200) + (hour * 3600) + (minute * 60) + second);
    	} else {
    		return 0;
    	}
    }
    
    public long getMillisFromMidNight() {
    	return this.getSecondsFromMidNight() * 1000;
    }

    public String toString() {
    	
    	StringBuffer sb = new StringBuffer();
    	
    	if(this.hour < 10)
    		sb.append("0"+String.valueOf(this.hour)+":");
    	else
    		sb.append(String.valueOf(this.hour)+":");
    	
    	if(this.minute < 10)
    		sb.append("0"+String.valueOf(this.minute)+":");
    	else
    		sb.append(String.valueOf(this.minute)+":");
    	
    	if(this.second < 10)
    		sb.append("0"+String.valueOf(this.second));
    	else
    		sb.append(String.valueOf(this.second));
        
    	return sb.toString();
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o instanceof Time) {
        	Time other = (Time) o;
            return this.getSecondsFromMidNight() == other.getSecondsFromMidNight();
        }
        return false;
    }
	
}


