package uoc.tfg.model.marketdata;

import java.util.Date;
import java.util.Vector;


public class TimeSeries {
	
	/**
	 * Equity id
	 */
	private final Instrument instrument;
	
	private final Vector<UnitaryData> dividends = new Vector<UnitaryData>();
	
	private final Vector<UnitaryData> splits = new Vector<UnitaryData>();
	
	private final Vector<UnitaryData> ampliations = new Vector<UnitaryData>();
	
	private final Vector<UnitaryData> hitoricalData = new Vector<UnitaryData>();
	
	private final Vector<UnitaryData> sessionData = new Vector<UnitaryData>();
	
	private boolean sessionUpdated = false;
	
	private boolean historicalUpdated = false;
	
	/**
	 * If the timeseries has an historical condition to be traded
	 * during the session.
	 */
	private boolean isTradable = true;
	
	
	/**
	 * 
	 * @param resolution
	 * @throws ClassNotFoundException 
	 */
	public TimeSeries(Instrument instrument) {
		this.instrument = instrument;
	}
	
	
	//GETTERS
	
	/**
	 * 
	 * @return
	 */
	public Instrument getInstrument() {
		return this.instrument;
	}
	/**
	 * 
	 * @return
	 */
	public UnitaryData readHistoricalSeries() {
		return this.hitoricalData.lastElement();
	}
	/**
	 * 
	 * @return
	 */
	public UnitaryData readSessionSeries() {
		return this.sessionData.lastElement();
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean isSessionUpdated() {
		return this.sessionUpdated;
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean isHistoriacalUpdated() {
		return this.historicalUpdated;
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean isTradable() {
		return this.isTradable;
	}
	
	public Vector<UnitaryData> getHistoricalData() {
		return this.hitoricalData;
	}
	
	public Vector<UnitaryData> getSessionData() {
		return this.sessionData;
	}
	
	public Vector<UnitaryData> getDividends() {
		return this.dividends;
	}
	
	public Vector<UnitaryData> getSplits() {
		return this.splits;
	}
	
	public Vector<UnitaryData> getAmpliations() {
		return this.ampliations;
	}
	
	public UnitaryData getHistoricalUnitaryData(Date time) {
		return this.getUnitaryData(this.hitoricalData, time);
	}
	
	public UnitaryData getSessionUnitaryData(Date time) {
		return this.getUnitaryData(this.sessionData, time);
	}
	
	public UnitaryData getDividend(Date time) {
		return this.getUnitaryData(this.dividends, time);
	}
	
	public UnitaryData getSplit(Date time) {
		return this.getUnitaryData(this.splits, time);
	}
	
	public UnitaryData getAmpliation(Date time) {
		return this.getUnitaryData(this.ampliations, time);
	}
	
	/**
	 * Get bar data by time stamp
	 * @param time
	 * @return
	 */
	public UnitaryData getUnitaryData(Vector<UnitaryData> vector, Date time) {
		if(!vector.isEmpty()) {
			UnitaryData data;
			boolean found = false;
			int positionBotton = 0;
			int position = (vector.size()-positionBotton)/2;
			data = vector.get(position);
		
			while(!found) {
				if(data.getOpentime() == time) {
					found = true;
				} else if(data.getOpentime().before(time)) {
					position -= (position-positionBotton)/2;
					data = vector.get(position);
				} else if(data.getOpentime().after(time)) {
					positionBotton = position;
					position += position/2;
					data = vector.get(position);
				}
			}
		
			return data;
		}
		
		return null;
	}
	

	
	//SETTERS

	public void setTradable(boolean tradable) {
		this.isTradable = tradable;
	}
	
	/**
	 * 
	 * @param data
	 */
	public void updateSessionData(UnitaryData data) {
		if(data != null) {
			this.sessionData.add(data);
			this.sessionUpdated = true;
		} else {
			this.sessionUpdated = false;
		}
	}
	
	/**
	 * 
	 * @param data
	 */
	public void updateHistoricalData(UnitaryData data) {
		if(data != null) {
			this.hitoricalData.add(data);
			this.historicalUpdated = true;
		} else {
			this.historicalUpdated = false;
		}
	}
	
	public float computeAdjustFactor(Date date) {
		
		
		return 0;		
	}
	
}