package uoc.tfg.model.marketdata;

import java.util.Date;

import uoc.tfg.model.Interval;


public class UnitaryBarData implements UnitaryData {
	
	private final Interval resolution;
	
	private Date openTime;
	
	private float open;
	
	private float maximun;
	
	private float minimun;
	
	private float close;
	
	private float openAdjusted;
	
	private float maximunAdjusted;
	
	private float minimunAdjusted;
	
	private float closeAdjusted;
	
	private long volume;
	
	/**
	 * Constructor
	 * 
	 * @param resolution
	 * @param time
	 * @param open
	 * @param maximun
	 * @param minimun
	 * @param close
	 * @param volume
	 */
	public UnitaryBarData(Interval resolution, Date time, float open, 
			float maximun, float minimun, float close, long volume) {
		this.resolution = resolution;
		this.openTime = time;
		this.open = open;
		this.maximun = maximun;
		this.minimun = minimun;
		this.close = close;
		this.openAdjusted = 0;
		this.maximunAdjusted = 0;
		this.minimunAdjusted = 0;
		this.closeAdjusted = 0;
		this.volume = volume;
	}
	
	public UnitaryBarData(Interval resolution, Date time) {
		this.resolution = resolution;
		this.openTime = time;
		this.open = 0;
		this.maximun = 0;
		this.minimun = 0;
		this.close = 0;
		this.openAdjusted = 0;
		this.maximunAdjusted = 0;
		this.minimunAdjusted = 0;
		this.closeAdjusted = 0;
		this.volume = 0;
	}
	
	public UnitaryBarData() {
		this.resolution = null;
		this.openTime = null;
		this.open = 0;
		this.maximun = 0;
		this.minimun = 0;
		this.close = 0;
		this.openAdjusted = 0;
		this.maximunAdjusted = 0;
		this.minimunAdjusted = 0;
		this.closeAdjusted = 0;
		this.volume = 0;
	}
	
	/**
	 * GETTERS
	 * 
	 * Get resolution
	 */
	public Interval getResolution() {
		return this.resolution;
	}
	/**
	 * 
	 * @param type
	 * @return
	 */
	@Override
	public float getPrice(PriceType type) {
		switch(type) {
		case CLOSE:
			return this.close;
		case MAXIMUN:
			return this.maximun;
		case MINIMUN:
			return this.minimun;
		case OPEN:
			return this.open;
		default:
			throw new IllegalArgumentException();
		}
	}
	/**
	 * Get time stamp
	 * 
	 * @return
	 */
	@Override
	public Date getOpentime() {
		return this.openTime;
	}
	/**
	 * 
	 * @return
	 */
	@Override
	public Date getClosetime() {
		return new Date(this.openTime.getTime() + resolution.getMilliseconds());
	}
	/**
	 * Get open
	 */
	public float getOpen() {
		return this.open;
	}
	/**
	 * Get maximun
	 */
	public float getMaximun() {
		return this.maximun;
	}
	/**
	 * Get minimun
	 */
	public float getMinimun() {
		return this.minimun;
	}
	/**
	 * Get close
	 */
	public float getClose() {
		return this.close;
	}
	/**
	 * Get open adjusted
	 */
	public float getOpenAdjusted() {
		return this.openAdjusted;
	}
	/**
	 * Get maximun Adjusted
	 */
	public float getMaximunAdjusted() {
		return this.maximunAdjusted;
	}
	/**
	 * Get minimun Adjusted
	 */
	public float getMinimunAdjusted() {
		return this.minimunAdjusted;
	}
	/**
	 * Get close Adjusted
	 */
	public float getCloseAdjusted() {
		return this.closeAdjusted;
	}
	/**
	 * Get volumen
	 */
	public long getVolumen() {
		return this.volume;
	}
	
	
	/**
	 * SETTERS
	 * 
	 * Set time stamp
	 */
	public void setOpentime(Date time) {
		this.openTime = time;
	}
	/**
	 * Set open
	 */
	public void setOpen(float open) {
		this.open = open;
	}
	/**
	 * Set maximun
	 */
	public void setMaximun(float maximun) {
		this.maximun = maximun;
	}
	/**
	 * Set minimun
	 */
	public void setMinimun(float minimun) {
		this.minimun = minimun;
	}
	/**
	 * Set close
	 */
	public void setClose(float close) {
		this.close = close;
	}
	/**
	 * Set open Adjusted
	 */
	public void setOpenAdjusted(float open) {
		this.openAdjusted = open;
	}
	/**
	 * Set maximun Adjusted
	 */
	public void setMaximunAdjusted(float maximun) {
		this.maximunAdjusted = maximun;
	}
	/**
	 * Set minimun Adjusted
	 */
	public void setMinimunAdjusted(float minimun) {
		this.minimunAdjusted = minimun;
	}
	/**
	 * Set close Adjusted
	 */
	public void setCloseAdjusted(float close) {
		this.closeAdjusted = close;
	}
	/**
	 * Set volume
	 */
	public void setVolume(long volume) {
		this.volume = volume;
	}
	
	public synchronized void adjustPrices(float factor) {
		this.openAdjusted = this.open * factor;
		this.maximunAdjusted = this.maximun * factor;
		this.minimunAdjusted = this.minimun * factor;
		this.closeAdjusted = this.close * factor;
	}
	/**
	 * Text representation of a unitary bardata object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		
		sb.append(this.openTime.toString());
		sb.append("\t");
		sb.append(this.open);
		sb.append("\t");
		sb.append(this.maximun);
		sb.append("\t");
		sb.append(this.minimun);
		sb.append("\t");
		sb.append(this.close);
		sb.append("\t");
		sb.append(this.volume);
		
		return sb.toString();
	}

}
