package uoc.tfg.model.marketdata.lightstreamer;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import uoc.tfg.model.marketdata.UnitaryTickData;

import com.lightstreamer.ls_client.UpdateInfo;


public class FeedHandlerLS extends ConcurrentLinkedQueue<UpdateInfo> implements Runnable {
	
	private static final long serialVersionUID = 8036215838525088363L;
	private final SimpleDateFormat sdf1 = new SimpleDateFormat("dd/MM/yyHH:mm:ss");
	private final SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private final ExecutorService feedHandlerExecutor;
	private IDataPullerLS dataPuller;
	
	
	public FeedHandlerLS(IDataPullerLS dataPuller) {
		this.feedHandlerExecutor = Executors.newSingleThreadExecutor();
		this.dataPuller = dataPuller;
	}
	
	public synchronized void start() {
		this.feedHandlerExecutor.execute(new Thread(this));
	}

	
	@Override
	public void run() {
		while(true) {
			if(this.peek() != null) {
				this.pollQueue();
			} else {
				this.waitPollingQueue();
			}
		}
	}
	
	public synchronized void waitPollingQueue() {
		try {
			wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public synchronized void updateQueue(UpdateInfo update) {
		this.add(update);
		notify();
	}
	
	public synchronized void pollQueue() {
		this.pushData(this.poll());
	}
	
	public synchronized void pushData(UpdateInfo update) {
		Date date = null;
		try {
			date = sdf1.parse(update.getNewValue(7));
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Long volume = new Long(0);
		if(update.getNewValue(6) != null)  volume = Long.parseLong(update.getNewValue(6));
		UnitaryTickData td = null;
		if (update.getNewValue(6) != "") {
			td = new UnitaryTickData(
					update.getItemName(),
					date,
					Float.parseFloat(update.getNewValue(1)),
					volume);
		} else {
			td = new UnitaryTickData(
					update.getItemName(),
					date,
					Float.parseFloat(update.getNewValue(1)),
					0);
		}
		
		this.dataPuller.update(td, update);
	}

}
