/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 02/12/2013
 *
 */
package uoc.tfg.model.news;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.URL;
import java.net.URLConnection;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.TextField;
import org.apache.tika.exception.TikaException;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.parser.html.BoilerpipeContentHandler;
import org.apache.tika.parser.html.HtmlParser;
import org.apache.tika.sax.BodyContentHandler;
import org.apache.tika.sax.Link;
import org.apache.tika.sax.LinkContentHandler;
import org.xml.sax.SAXException;

import uoc.tfg.model.DateParser;
import uoc.tfg.model.document.DocumentField;

/*
 *
 */
public class WebContentSimpleParser implements IWebContentParser, Serializable {

	private static final long serialVersionUID = 8918258187390196304L;
	
	private final static String[] dateMetadata = {
		"article:published_time",
		"date",
		"DC.date.issue",
		"dateCreated",
		"DC.date.issued",
		"article:published_time"};
	private final static String[] dateModifiedMetadata = {
		"article:modified_time",
		"last-modified"};
	
	
	/**
	 * @throws IOException 
	 * 
	 */
	@Override
	public InputStream getImputStream(URL url) throws IOException {
		InputStream inStream = null;
		URLConnection connection = url.openConnection();
		inStream = connection.getInputStream();
		
		return inStream;
	}
	
	
	/**
	 * @param url
	 * @return
	 * @throws IOException 
	 * @throws ParseException 
	 * @throws TikaException 
	 * @throws SAXException 
	 */
	@Override
	public synchronized WebContent getWebContent(URL url) throws IOException, ParseException {
		
		InputStream inputStream = this.getImputStream(url);
		Metadata metadata = new Metadata();
		metadata.set(Metadata.RESOURCE_NAME_KEY, url.getRef());
		
		Parser parser = new AutoDetectParser();
		HtmlParser htmlparser = new HtmlParser();
        BodyContentHandler contentHandler = new BodyContentHandler();
        BoilerpipeContentHandler boilerHandler = new BoilerpipeContentHandler(contentHandler);
        
        ParseContext context = new ParseContext();
        context.set(Parser.class, parser);
        
        try {
			parser.parse(inputStream, boilerHandler, metadata, context);
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TikaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			this.getImputStream(url).close();
		}
        
        WebContent wc = null;
        
        if(metadata.get(Metadata.CONTENT_TYPE).matches("(.*)html(.*)")){
        	
        	try {
				htmlparser.parse(inputStream, boilerHandler, metadata, context);
			} catch (SAXException | TikaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
        	String date = null;
        	for(String s: dateMetadata) {
        		date = metadata.get(s);
        		if(date != null) break;
        	}
        	if(date == null) {
        		date = DateParser.parseInputStream(inputStream);
        	}
        	wc = new WebContent(
        			url,
            		metadata.get("title"),
            		WebContentFormat.HTML,
            		DateParser.parse(date),
            		boilerHandler.getTextDocument().getContent());
        }
        
		return wc;
	}
	
	@Override
	public Document getDocument(URL url) throws IOException, SAXException, TikaException, ParseException {
		
		InputStream inputStream = this.getImputStream(url);
		Metadata metadata = new Metadata();
		metadata.set(Metadata.RESOURCE_NAME_KEY, url.getRef());
		
		Parser parser = new AutoDetectParser();
		HtmlParser htmlparser = new HtmlParser();
        BodyContentHandler contentHandler = new BodyContentHandler();
        BoilerpipeContentHandler boilerHandler = new BoilerpipeContentHandler(contentHandler);
        
        ParseContext context = new ParseContext();
        context.set(Parser.class, parser);
        
        try {
			parser.parse(inputStream, boilerHandler, metadata, context);
		} finally {
			this.getImputStream(url).close();
		}

        Document doc = new Document();
        
        if(metadata.get(Metadata.CONTENT_TYPE).matches("(.*)html(.*)")) {
        	
        	String date = null;
        	for(String s: dateMetadata) {
        		date = metadata.get(s);
        		if(date != null) break;
        	}
        	if(date == null) {
        		date = DateParser.parseInputStream(inputStream);
        	}
        	
        	htmlparser.parse(this.getImputStream(url), boilerHandler, metadata, context);
        	
            if(url != null) doc.add(new Field(DocumentField.URL.toString(), url.toString(), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.URL.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("title") != null) doc.add(new Field(DocumentField.TITLE.toString(), metadata.get("title"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.TITLE.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("author") != null) doc.add(new Field(DocumentField.AUTHOR.toString(), metadata.get("author"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.AUTHOR.toString(), "", TextField.TYPE_STORED));
            if (date != null) doc.add(new Field(DocumentField.DATE.toString(), date, TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.DATE.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("keywords") != null) doc.add(new Field(DocumentField.KEYWORD.toString(), metadata.get("keywords"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.KEYWORD.toString(), "", TextField.TYPE_STORED));
            if (boilerHandler.getTextDocument().getContent() != null) doc.add(new Field(DocumentField.CONTENT.toString(), boilerHandler.getTextDocument().getContent(), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.CONTENT.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("subject") != null) doc.add(new Field(DocumentField.SUBJECT.toString(), metadata.get("subject"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.SUBJECT.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("description") != null) doc.add(new Field(DocumentField.DESCRIPTION.toString(), metadata.get("description"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.DESCRIPTION.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("Content-Type") != null) doc.add(new Field(DocumentField.FORMAT.toString(), metadata.get("Content-Type"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.FORMAT.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("lang") != null) doc.add(new Field(DocumentField.LANGUAGE.toString(), metadata.get("lang"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.LANGUAGE.toString(), "", TextField.TYPE_STORED));
            if (metadata.get("locality") != null) doc.add(new Field(DocumentField.LOCATION.toString(), metadata.get("locality"), TextField.TYPE_STORED));
            else doc.add(new Field(DocumentField.LOCATION.toString(), "", TextField.TYPE_STORED));
        }
        
        return doc;
	}


	/**
	 * 
	 */
	@Override
	public List<URL> findLinksInContent(URL url) throws IOException {
		
		List<URL> urls = new ArrayList<URL>();
		Metadata metadata = new Metadata();	
        Parser parser = new AutoDetectParser();
        LinkContentHandler linkHandler = new LinkContentHandler();

        ParseContext context = new ParseContext();
        context.set(Parser.class, parser);
        
        try {
			parser.parse(this.getImputStream(url), linkHandler, metadata, context);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TikaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			this.getImputStream(url).close();
		}
        
        List<Link> links = linkHandler.getLinks();
        
        for(Link l: links) {
        	Pattern pattern = Pattern.compile("http(s){0,1}://(.)*?\"");
        	if(l.toString().matches("(.)*"+pattern.pattern()+"(.)*")){
        		Matcher m = pattern.matcher(l.toString());
        		m.find();
        		urls.add(new URL(m.group(0).split("\"")[0]));
        	}
        	
        }
        
		return urls;
	}

	/**
	 * 
	 */
	@Override
	public int getPriority(WebContent content) {
		if(content.getTitle().matches("(.)*[Ii][Bb][Ee][Xx](.)*")) return 1;
		else return 0;
	}	
}
