/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 02/12/2013
 *
 */
package uoc.tfg.model.news.webcrawler;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;


public class WebCrawlingURL implements Serializable, Comparable<WebCrawlingURL> {

	private static final long serialVersionUID = -206575187299271944L;
	
	private String name;
	private String description;
	private URL url;
	private int depth = 0;
	private boolean visited = false;
	private boolean allowedToVisit;
	private boolean checkedForPermision = false;
	private int priority = 1;
	
	public WebCrawlingURL() throws MalformedURLException {
		this(null, null, "http://");
	}
	
	public WebCrawlingURL(String name, String description, String url) throws MalformedURLException {
		this(name, description, new URL(url));
	}
	
	public WebCrawlingURL(String name, String description, URL url) {
		this.name = name;
		this.description = description;
		this.url = url;
	}
	
	public WebCrawlingURL(URL url, int priority, int depth) {
		this.url = url;
		this.priority = priority;
		this.depth = depth;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the url
	 */
	public URL getUrl() {
		return url;
	}
	
	public String getUrlText() {
		return url.toString();
	}

	/**
	 * @param url the url to set
	 */
	public void setUrl(URL url) {
		this.url = url;
	}

	/**
	 * @return the depth
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @param depth the depth to set
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}

	/**
	 * @return the visited
	 */
	public boolean isVisited() {
		return visited;
	}

	/**
	 * @param visited the visited to set
	 */
	public void setVisited(boolean visited) {
		this.visited = visited;
	}

	/**
	 * @return the priority
	 */
	public int getPriority() {
		return priority;
	}

	/**
	 * @param priority the priority to set
	 */
	public void setPriority(int priority) {
		this.priority = priority;
	}

	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(WebCrawlingURL o) {
		if(o.getPriority() > this.priority) return 1;
		else if(o.getPriority() == this.priority) return 0;
		else return -1;
	}

	/**
	 * @return the allowedToVisit
	 */
	public boolean isAllowedToVisit() {
		return allowedToVisit;
	}

	/**
	 * @param allowedToVisit the allowedToVisit to set
	 */
	public void setAllowedToVisit(boolean allowedToVisit) {
		this.allowedToVisit = allowedToVisit;
	}

	/**
	 * @return the checkedForPermision
	 */
	public boolean isCheckedForPermision() {
		return checkedForPermision;
	}

	/**
	 * @param checkedForPermision the checkedForPermision to set
	 */
	public void setCheckedForPermision(boolean checkedForPermision) {
		this.checkedForPermision = checkedForPermision;
	}	

}
