/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 21/12/2013
 *
 */
package uoc.tfg.model.news.webfeed;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedDeque;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.lucene.document.Document;

import uoc.tfg.exception.WebFeedException;
import uoc.tfg.model.Status;
import uoc.tfg.model.news.IWebContentParser;
import uoc.tfg.model.news.IWebContentPuller;
import uoc.tfg.model.news.WebContent;
import uoc.tfg.model.news.WebContentSimpleParser;

import com.sun.syndication.feed.synd.SyndEntry;
import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.XmlReader;

/*
 *
 */
public class WebFeedReader extends Observable implements IWebFeedReader {

	private static final long serialVersionUID = 8455129738033346595L;
	private WebFeedSubscription subscription;
	private Status status = Status.STOPPED;
	private IWebContentParser contentParser;
	private IWebContentPuller contentPuller;
	private List<SyndEntry> allEntries;
	private Queue<SyndEntry> lastEntries;
	private ExecutorService executor;
	
	
	public WebFeedReader(WebFeedSubscription subscription, IWebContentPuller contentPuller) {
		super();
		this.subscription = subscription;
		this.addObserver(contentPuller);
		this.contentParser = new WebContentSimpleParser();
		this.contentPuller = contentPuller;
		this.allEntries = new ArrayList<SyndEntry>();
		this.lastEntries = new ConcurrentLinkedDeque<SyndEntry>();
		this.executor = Executors.newSingleThreadScheduledExecutor();
	}
	
	private synchronized void getRssFeeds(List<WebFeedURL> urls) throws Exception { 
		for(WebFeedURL url: urls) {
			HttpURLConnection con = (HttpURLConnection)url.getUrl().openConnection();
			SyndFeedInput input = new SyndFeedInput();
			SyndFeed feed = input.build(new XmlReader(con));
			List<SyndEntry> entries = feed.getEntries();
			Iterator<SyndEntry> itEntries = entries.iterator();
			while (itEntries.hasNext()) {
				SyndEntry entry = itEntries.next();
				if(entry.getPublishedDate() != null && 
						entry.getPublishedDate().after(url.getLastPoll())){
					this.addEntry(entry);
					URL link = new URL(entry.getUri());
					if(this.urlIsVisitable(link)) {
						WebContent wc = this.contentParser.getWebContent(link);
						Document doc = this.contentParser.getDocument(link);
						this.contentPuller.update(wc);
				        this.contentPuller.update(doc);
					}
				}
			}
			url.setLastPoll();
		}
	}

	/**
	 * @param link
	 * @return
	 */
	private boolean urlIsVisitable(URL url) {
		// TODO Auto-generated method stub
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.uoc.tfg.news.webfeed.IWebFeedReader#startWebFeedReader()
	 */
	@Override
	public synchronized void startWebFeedReader() throws WebFeedException {
		if(this.status.getValue() == Status.STOPPED.getValue()) {
			this.setStatus(Status.RUNNING);
			this.executor.execute(this);
		} else {
			throw new WebFeedException(WebFeedException.READER_RUNNING);
		}
	}

	/* (non-Javadoc)
	 * @see edu.uoc.tfg.news.webfeed.IWebFeedReader#pauseWebFeedReader()
	 */
	@Override
	public synchronized void pauseWebFeedReader() {
		this.setStatus(Status.PAUSED);
	}

	/* (non-Javadoc)
	 * @see edu.uoc.tfg.news.webfeed.IWebFeedReader#resumeWebFeedReader()
	 */
	@Override
	public synchronized void resumeWebFeedReader() {
		this.setStatus(Status.RUNNING);
	}

	/* (non-Javadoc)
	 * @see edu.uoc.tfg.news.webfeed.IWebFeedReader#stopWebFeedReader()
	 */
	@Override
	public synchronized void stopWebFeedReader() {
		this.setStatus(Status.STOPPED);
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		while(this.status.isnot(Status.STOPPED)) {
			try {
				this.getRssFeeds(subscription.getUrls());
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			
			while(this.status.is(Status.PAUSED))	{
				try {
					Thread.sleep(this.subscription.getPollInterval());
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			try {
				Thread.sleep(this.subscription.getPollInterval());
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	@Override
	public String getName() {
		return this.subscription.getName();
	}

	/**
	 * @return the subscription
	 */
	public WebFeedSubscription getSubscription() {
		return subscription;
	}

	/**
	 * @param subscription the subscription to set
	 */
	public void setSubscription(WebFeedSubscription subscription) {
		this.subscription = subscription;
	}

	/**
	 * @return the status
	 */
	@Override
	public Status getStatus() {
		return this.status;
	}

	/**
	 * @param status the status to set
	 */
	public synchronized void setStatus(Status status) {
		this.status = status;
		this.setChanged();
		this.notifyObservers();
	}
	
	/**
	 * @return the allEntries
	 */
	public List<SyndEntry> getAllEntries() {
		return allEntries;
	}

	/**
	 * @param allEntries the allEntries to set
	 */
	public void setAllEntries(List<SyndEntry> allEntries) {
		this.allEntries = allEntries;
	}

	/**
	 * @return the lastEntries
	 */
	@Override
	public Queue<SyndEntry> getLastEntries() {
		return lastEntries;
	}

	/**
	 * @param lastEntries the lastEntries to set
	 */
	public void setLastEntries(Queue<SyndEntry> lastEntries) {
		this.lastEntries = lastEntries;
	}

	/**
	 * @param entry
	 */
	private synchronized void addEntry(SyndEntry entry) {
		this.allEntries.add(entry);
		this.lastEntries.add(entry);
		this.setChanged();
		this.notifyObservers();
	}

}

