/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 08/01/2014
 *
 */
package uoc.tfg.model.textanalysis;

import java.io.IOException;
import java.io.Reader;
import java.util.HashSet;
import java.util.Set;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.Tokenizer;
import org.apache.lucene.analysis.core.LowerCaseFilter;
import org.apache.lucene.analysis.core.StopFilter;
import org.apache.lucene.analysis.es.SpanishAnalyzer;
import org.apache.lucene.analysis.standard.StandardTokenizer;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.apache.lucene.analysis.util.CharArraySet;
import org.apache.lucene.util.Version;

/**
 * 
 * TextAnalyser
 */
public class SpanishTokenizer extends Analyzer implements ITokenizer {

	private Analyzer analyzer = new SpanishAnalyzer(Version.LUCENE_46);
	private final CharArraySet stopWorlds = StopWord.getGoogleSpanish();
	
	public SpanishTokenizer() {
		super();
	}
		
	
	@Override
	protected TokenStreamComponents createComponents(String n, Reader r) {
		Tokenizer tokenizer = new StandardTokenizer(Version.LUCENE_46, r);
		TokenFilter caseFilter = new LowerCaseFilter(Version.LUCENE_46, tokenizer);
		TokenFilter stopWorldFilter = new StopFilter(Version.LUCENE_46, caseFilter, stopWorlds);
		
		//TokenFilter filter = new SpanishLightStemFilter(stopWorldFilter);
		return new TokenStreamComponents(tokenizer, stopWorldFilter);
	}

	@Override
	public String getToken(String world) throws IOException {

		TokenStream ts = null;
		CharTermAttribute cattr = null;
		String token = null;
				
		ts = analyzer.tokenStream(null, world);
		cattr= ts.addAttribute(CharTermAttribute.class);
		ts.reset();
		while (ts.incrementToken()) {
			token = cattr.toString();
		}
		ts.end();
		ts.close();
		
		return token;
	}

	@Override
	public Set<String> getTokens(String text) throws IOException {
		
		TokenStream ts = null;
		CharTermAttribute cattr = null;
		Set<String> tokens = new HashSet<String>();
				
		ts = analyzer.tokenStream(null, text);
		cattr= ts.addAttribute(CharTermAttribute.class);
		ts.reset();
		while (ts.incrementToken()) {
			tokens.add(cattr.toString());
		}
		ts.end();
		ts.close();
		
		return tokens;
	}


	@Override
	public TokenRepresentation getTokenRepresentation(TokenCollection collection) {
		// TODO Auto-generated method stub
		return null;
	}


	

}
